{*******************************************************************}
{								    }
{	HEComp Unit. Enthlt die Komponente THighEdit	            }
{								    }
{	Name:  HEComp.pas 					    }
{	Autoren: Jan Hamann/ Martin Kammann                         }
{	Datum: 5.4.94						    }
{								    }
{*******************************************************************}

unit HEComp;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  StdCtrls, Forms, Dialogs, HighEdit, DlgTab;

{$R *.RES}


const
  id_HighEdit        = $CA0;
  id_Status          = $CA5;
  id_Tab             = $CA6;
  id_Ruler           = $CA7;
  id_Icon            = $CA8;

  ids_Title          = 1;
  ids_NoMemory       = 2;
  ids_LowMemory      = 3;
  ids_NoSave         = 4;
  ids_NoLoad         = 5;
  ids_NoStart        = 6;
  ids_NoBlockMark    = 7;
  ids_NoTmpOpen      = 8;
  ids_NoTmpClose     = 9;
  ids_NoHEFile       = 10;
  ids_NoTmpSpace     = 11;
  ids_Untitled       = 12;
  ids_SearchNotFound = 13;
  ids_SearchError    = 14;
  ids_CloseSave      = 15;
  ids_WrongVersion   = 16;
  ids_NoRTFFile      = 17;
  ids_NoFont         = 18;
  ids_NoPrinter      = 19;
  ids_NoWriteAccess  = 21;
  ids_ReplaceFile    = 22;
  ids_LowResources   = 23;
  ids_NoTableInsert  = 24;

  hem_KeyDown        = 64;
  hem_KeyUp          = 65;
  hem_Char           = 66;
  hem_SysKeyDown     = 67;
  hem_SysKeyUp       = 68;
  hem_SysChar        = 69;
  hem_LButtonDown    = 72;
  hem_MButtonDown    = 73;
  hem_RButtonDown    = 74;
  hem_LButtonUp      = 75;
  hem_MButtonUp      = 76;
  hem_RButtonUp      = 77;
  hem_LButtonDblClk  = 78;
  hem_MButtonDblClk  = 79;
  hem_RButtonDblClk  = 80;
  hem_MouseMove      = 81;
  hem_Click          = 82;

  he_GetNotifyWParam = wm_User + 317;
  he_GetNotifyLParam = wm_User + 318;

  HEFilter = 'HighEdit (*.hed)|*.hed|ANSI (*.txt)|*.txt|OEM (*.txt)|*.txt|' +
             'Rich Text Format (*.rtf)|*.rtf|Alle Dateien (*.*)|*.*';

type
  TUnitType = (utCentimeter, utInch);
  TShowControl = (scIconBar, scRuler, scTabulator, scStatusLine);
  TShowControls = Set of TShowControl;

type
  THighEdit = class(TWinControl)
    procedure ChangeStatusFont(Sender: TObject);
  private
    FBackColorBoxParamPtr   : PBackColorBoxParam;
    FBkBrush                : TBrush;
    FColorBoxParamPtr       : PColorBoxParam;
    FDblClickSelectWord     : Boolean;
    FDocInfoBoxParamPtr     : PDocInfoBoxParam;
    FDoSaveAs               : Boolean;
    FFieldInsertHandle      : HWnd;
    FFileBoxParamPtr        : PFileBoxParam;
    FFontBoxParamPtr        : PFontBoxParam;
    FFormatDocBoxParamPtr   : PFormatDocBoxParam;
    FHighEditHandle         : HWnd;
    FIconHandle             : HWnd;
    FParagraphBoxParamPtr   : PParagraphBoxParam;
    FPictureFileBoxParamPtr : PFileBoxParam;
    FPrintBoxParamPtr       : PPrintBoxParam;
    FPrintFoot              : Boolean;
    FPrintHead              : Boolean;
    FRedraw                 : Boolean;
    FRulerHandle            : HWnd;
    FSearchBoxParamPtr      : PSearchBoxParam;
    FStatusFont             : TFont;
    FStatusHandle           : HWnd;
    FTabHandle              : HWnd;
    FTextBorderStyle        : TBorderStyle;
    FTextSelectable         : Boolean;
    FUntitled               : Boolean;
    FUseIcon                : Boolean;
    FUseRuler               : Boolean;
    FUseStatus              : Boolean;
    FUseTab                 : Boolean;
    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FOnInitNew: TNotifyEvent;
    FOnPictureSelect: TNotifyEvent;
    procedure CreateHighEdit;
    procedure CreateIconBar;
    procedure CreateRuler;
    procedure CreateStatusLine;
    procedure CreateTabulator;
    procedure SetRedraw(Value: Boolean);
    function GetBkColor: TColor;
    function GetFileName: TFileName;
    function GetGrayBk: Boolean;
    function GetShowControls: TShowControls;
    function GetUnitType: TUnitType;
    function GetVersion: LongInt;
    procedure SetBkColor(Value: TColor);
    procedure SetFileName(Value: TFileName);
    procedure SetGrayBk(Value: Boolean);
    procedure SetShowControls(Value: TShowControls);
    procedure SetStatusFont(Value: TFont);
    procedure SetTextBorderStyle(Value: TBorderStyle);
    procedure SetTextSelectable(Value: Boolean);
    procedure SetUnitType(Value: TUnitType);
  protected
    procedure WMCommand(var Msg : TWMCommand);   message WM_COMMAND;
    procedure WMSize(var Msg: TMessage);         message wm_Size;
    procedure WMMove(var Msg: TMessage);         message wm_Move;
    procedure WMEnterIdle(var Msg: TMessage);    message wm_EnterIdle;
    procedure WMEndSession(var Msg: TMessage);   message wm_EndSession;
    procedure WMSetFocus(var Msg: TMessage);     message wm_SetFocus;
    procedure WMWinIniChange(var Msg: TMessage); message wm_WinIniChange;
    procedure WMCtlColor(var Msg: TMessage);     message wm_CtlColor;
    procedure NewWindowTitle;
    {******************** HighEdit-Messages ********************}
    {Messages EditWindow to ParentWindow}
    procedure HEMAlign(var Msg: TMessage);             virtual;
    procedure HEMChar(var Msg: TMessage);              virtual;
    procedure HEMClick(var Msg: TMessage);             virtual;
    procedure HEMCursorPos(var Msg: TMessage);         virtual;
    procedure HEMDblClk(var Msg: TMessage);            virtual;
    procedure HEMEndPF(var Msg: TMessage);             virtual;
    procedure HEMFileExists(var Msg: TMessage);        virtual;
    procedure HEMFont(var Msg: TMessage);              virtual;
    procedure HEMFontSize(var Msg: TMessage);          virtual;
    procedure HEMFormatPF(var Msg: TMessage);          virtual;
    procedure HEMHScroll(var Msg: TMessage);           virtual;
    procedure HEMInitNew(var Msg: TMessage);           virtual;
    procedure HEMInsert(var Msg: TMessage);            virtual;
    procedure HEMInsertPF(var Msg: TMessage);          virtual;
    procedure HEMKeyDown(var Msg: TMessage);           virtual;
    procedure HEMKeyUp(var Msg: TMessage);             virtual;
    procedure HEMKillFocus(var Msg: TMessage);         virtual;
    procedure HEMLButtonDown(var Msg: TMessage);       virtual;
    procedure HEMLButtonUp(var Msg: TMessage);         virtual;
    procedure HEMMButtonUp(var Msg: TMessage);         virtual;
    procedure HEMMButtonDown(var Msg: TMessage);       virtual;
    procedure HEMMessageText(var Msg: TMessage);       virtual;
    procedure HEMMouseMove(var Msg: TMessage);         virtual;
    procedure HEMNewPageFragment(var Msg: TMessage);   virtual;
    procedure HEMPrintFoot(var Msg: TMessage);         virtual;
    procedure HEMPrintHead(var Msg: TMessage);         virtual;
    procedure HEMRButtonUp(var Msg: TMessage);         virtual;
    procedure HEMRButtonDown(var Msg: TMessage);       virtual;
    procedure HEMSelectPF(var Msg: TMessage);          virtual;
    procedure HEMSelectPicture(var Msg: TMessage);     virtual;
    procedure HEMSetFocus(var Msg: TMessage);          virtual;
    procedure HEMSetHScrollPos(var Msg: TMessage);     virtual;
    procedure HEMSetHScrollRange(var Msg: TMessage);   virtual;
    procedure HEMSetVScrollPos(var Msg: TMessage);     virtual;
    procedure HEMSetVScrollRange(var Msg: TMessage);   virtual;
    procedure HEMSizePF(var Msg: TMessage);            virtual;
    procedure HEMSpacing(var Msg: TMessage);           virtual;
    procedure HEMStyle(var Msg: TMessage);             virtual;
    procedure HEMTabModify(var Msg: TMessage);         virtual;
    procedure HEMTabWidth(var Msg: TMessage);          virtual;
    procedure HEMVScroll(var Msg: TMessage);           virtual;
    {Messages IconBar}
    function  IBMGetAlign: Word;                                                 virtual;
    function  IBMGetColor: TColorRef;                                            virtual;
    function  IBMGetColorLogFont(APointer: PColorLogFont): Boolean;              virtual;
    function  IBMGetDisplay: Boolean;                                            virtual;
    function  IBMGetSpacing: Word;                                               virtual;
    function  IBMGetStyle: Word;                                                 virtual;
    function  IBMGetTab: Word;                                                   virtual;
    function  IBMHideDropDown: Boolean;                                          virtual;
    function  IBMSetAlign(AAlign: Word): Boolean;                                virtual;
    function  IBMSetColorLogFont(AFlag: Byte; APointer: PColorLogFont): Boolean; virtual;
    function  IBMSetDisplay(ASwitch: Boolean): Boolean;                          virtual;
    function  IBMSetFontSize(ASize: Word): Boolean;                              virtual;
    function  IBMSetSpacing(ASpacing: Word): Boolean;                            virtual;
    function  IBMSetStyle(AStyle: Word): Boolean;                                virtual;
    function  IBMSetTab(ATab: Word): Boolean;                                    virtual;
    function  IBMEnableScreenFonts(ASwitch: Boolean): Boolean;                   virtual;
    function  IBMIsScreenFont: Boolean;                                          virtual;
    {Messages IconBar to ParentWindow}
    procedure IBNAlign(var Msg: TMessage);                              virtual;
    procedure IBNColor(var Msg: TMessage);                              virtual;
    procedure IBNDblClk(var Msg: TMessage);                             virtual;
    procedure IBNDisplay(var Msg: TMessage);                            virtual;
    procedure IBNFontName(var Msg: TMessage);                           virtual;
    procedure IBNFontSize(var Msg: TMessage);                           virtual;
    procedure IBNSpacing(var Msg: TMessage);                            virtual;
    procedure IBNStyle(var Msg: TMessage);                              virtual;
    procedure IBNTab(var Msg: TMessage);                                virtual;
    {Messages Tabulator to ParentWindow}
    procedure TNIntInsert(var Msg: TMessage);          virtual;
    procedure TNIntUpdate(var Msg: TMessage);          virtual;
    procedure TNIntDelete(var Msg: TMessage);          virtual;
    {Messages Ruler to ParentWindow}
    procedure RNLButtonDblClk(var Msg: TMessage);      virtual;
    procedure RNRButtonDblClk(var Msg: TMessage);      virtual;
    {Messages StatusLine to ParentWindow}
    procedure IDStatus(var Msg: TMessage);                              virtual;
    {Error}
    function  HEError(wID: Word; AType: TMsgDlgType; AButtons: TMsgDlgButtons): Word; 
    {Fields}
    function  HEAddAllPFFlags(AFlag: Byte; ATypes: PWord): Boolean;
    function  HEAddFunctionPF(ATyp: Word; AName: PChar): Boolean;
    function  HEAddPFFlags(AFlag: Word): Boolean;
    function  HEChangeFunctionPF(AType: Word; ABuff: PChar): Boolean;   
    function  HEClearAllPFFlags(AFlag: Byte; ATypes: PWord): Boolean;
    function  HEClearPFFlags(AFlag: Word): Boolean;                     
    function  HECloseActivePF(ARepaint: Boolean): Boolean;
    function  HEDeletePF(ARepaint: Boolean): Boolean;                   
    function  HEGetFunctionTableCount: Longint;
    function  HEGetFunctionPFTyp(AIndex: word): Word;
    function  HEGetPFBkColor: TColorRef;                                
    function  HEGetPFData(ABuff: PChar): Word;                          
    function  HEGetPFDataLen: Word;
    function  HEGetPFFlags: Word;
    function  HEGetPFWidth: Longint;
    function  HELockFunctionPF(ATyp: Word): PChar;
    function  HEInsertFunctionPFParam(ATyp: word; AFlag: word; ABuff: PChar): Boolean;
    function  HEIsPFActive: Boolean;
    function  HEIsPFBkVisible: Boolean;
    function  HEIsPFFunctionVisible: Boolean;                          
    function  HEIsPFGhost: Boolean;                                    
    function  HENextPF(ASearch: Byte; ATypes: PWord): Word;
    function  HESetPFBkColor(AColor: TColorRef): Boolean;              
    function  HESetPFBkVisible(ASwitch: Boolean): Boolean;             
    function  HESetPFFunctionVisible(ASwitch: Boolean): Boolean;       
    function  HESetPFFlags(AFlag: Word): Boolean;
    function  HESetPFGhost(ASwitch: Boolean): Boolean;
    function  HESetPFWidth(AWidth: Word): Boolean;
    function  HEUnlockFunctionPF: Boolean;
    function  HESetAllPFFlags(AFlag: Byte; ATypes: PWord): Boolean;    
    function  HESetAllPFWidth(AWidth: Word; ATypes: PWord): Boolean;   
    {Bitmaps}
    function  HEGetPictureDim: Longint;                                
    function  HEGetPictureFlags: Word;                                 
    function  HEGetPictureOffset: Longint;                             
    function  HEIsPictureFrame: Boolean;
    function  HEInsertBitmap(ABitmap: HBitmap): Boolean;               
    function  HELoadBMP(AFlag: Word; AName: PChar): Boolean;           
    function  HELoadGIF(AFlag: Word; AName: PChar): Boolean;           
    function  HELoadPCX(AFlag: Word; AName: PChar): Boolean;           
    function  HELoadTGA(AFlag: Word; AName: PChar): Boolean;           
    function  HELoadTIF(AFlag: Word; AName: PChar): Boolean;
    function  HELoadWMF(AFlag: Word; AName: PChar): Boolean;
    function  HESetPictureDim(AWidth, AHeight: Word): Boolean;
    function  HESetPictureFlags(AFlags: Word): Boolean;
    function  HESetPictureFrame(AAsFrame: Boolean): Boolean;           
    function  HESetPictureOffset(AX, AY: Word): Boolean;               
    function  HELoadPictureExt(AFileType: Word; AName: PChar): Boolean;
    function  HELoadPicture: Boolean;
    {Cursor}
    function  HECursorEnd: Boolean;
    function  HECursorHome: Boolean;                                   
    function  HEGetCursor: PLongint;
    function  HESetCursor(ACol: Word; ARow:Longint): Boolean;
    function  HESetMouseCursor(ACursor: HCursor): Boolean;             
    function  HEGetCursorX: Longint;
    function  HEGetCursorY: Longint;
    {Print}
    function  HEGetPrinterCursorHeight: Longint;                       
    function  HEGetPrinterDC: HDC;                                     
    function  HEGetPrinterDim: Longint;                                
    function  HEGetPrinterDimX: Word;                                  
    function  HEGetPrinterDimY: Word;                                  
    function  HEGetPrinterIC: HDC;
    function  HEGetPrinterLineHeight: Longint;                         
    function  HEGetPrinterTextHeight: Longint;                         
    function  HEGetPrinterLineWidth: Longint;                          
    function  HEGetFragment: PTextFragment;                            
    function  HEGetInstancePrinter: Longint;                           
    function  HEGetInstancePrinterDevNames: THandle;
    function  HEGetInstancePrinterDevMode: THandle;                    
    function  HESetInstancePrinter(ADevNames, ADevMode: THandle): Boolean;
    function  HEPrintDocument(AMode: Word; APtr: PPrintDocParam): Boolean;
    function  HEPrintFragment(APtr: PTextFragment): Boolean;
    {Load}
    function  HELoadAnsi(AName: PChar): Boolean;                       
    function  HELoadData(AName: PChar): Boolean;
    function  HELoadOEM(AName: PChar): Boolean;
    function  HELoadRTF(AName: PChar): Boolean;
    function  HELoadExt(AFileType: Word; AName: PChar): Boolean;
    function  HELoad: Boolean;                                         
    {Append}
    function  HEAppendAnsi(AName: PChar): Boolean;
    function  HEAppendData(AName: PChar): Boolean;
    function  HEAppendOEM(AName: PChar): Boolean;
    function  HEAppendRTF(AName: PChar): Boolean;
    function  HEAppendExt(AFileType: Word; AName: PChar): Boolean;     
    function  HEAppend: Boolean;
    {Tabs}
    function  HEGetTab(ATabHandlePtr: PHandle): Boolean;               
    function  HEGetTabWidth: Longint;                                  
    function  HESetTab(ATabPtr: PTab): Boolean;
    function  HESetTabVisible(ASwitch: Boolean): Boolean;              
    function  HESetTabWidth(AWidth: Word): Boolean;                    
    function  HEDeleteTab(AOffset: longint): Boolean;
    function  HEMoveTab(APointer: PMoveTabParam): Boolean;             
    function  HEInsertTab(ATyp: Byte; APos: Word): Word;
    function  HEIsTabVisible: Boolean;                                 
    {Save}
    function  HESaveAnsi(AName: PChar): Boolean;                       
    function  HESaveData(AName: PChar): Boolean;                       
    function  HESaveOEM(AName: PChar): Boolean;                        
    function  HESaveRTF(AName: PChar): Boolean;                        
    function  HESaveExt(AFileType: Word; AName: PChar): Boolean;       
    function  HESave: Boolean;                                         
    {Dimensions}
    function  HEGetTextDim: Longint;                                   
    function  HEGetTextWidth: Word;                                    
    function  HEGetUnit: Longint;                                      
    function  HEMoveMargin(AParam: PMoveMarginParam): Boolean;
    function  HESetLeftMargin(AWidth: Word): Boolean;                  
    function  HESetTextDim(AWidth, AHeight: Word): Boolean;
    function  HESetMargins(AX, AY: Word): Boolean;
    function  HESetTextHeight(AHeight: Word): Boolean;
    function  HESetRBMargins(AX, AY: Word): Boolean;
    function  HESetTextWidth(AWidth: Word): Boolean;                   
    function  HESetTopMargin(AHeight: Word): Boolean;                  
    function  HESetUnit(AUnit: Word): Boolean;
    function  HEGetMargins: Longint;
    function  HEGetPhysicalDim: Longint;
    function  HEGetPhysicalMargins: Longint;
    function  HEGetRBMargins: Longint;                                 
    {Select}
    function  HESelectAll: Boolean;
    function  HESelectWord: Boolean;                                   
    function  HEGetTextSel: THandle;                                   
    function  HEIsTextSelectable: Boolean;
    function  HEReplaceSel(ABuff: PChar): Boolean;                     
    function  HESetSelection(ACol: Word; ARow:Longint): Boolean;       
    function  HESetSelectionEditable(AEdit: Boolean): Boolean;         
    function  HESetTextSelectable(Value: Boolean): Boolean;                            
    function  HEGetSelection: PLongint;                                
    function  HEGetSelectionStart: PLongint;                           
    function  HEGetMarkedText(ABuff: PChar; AMaxSize: Word): PChar;    
    function  HEGetSelectionX: Longint;                                
    function  HEGetSelectionY: Longint;                                
    function  HEGetSelectionStartX: Longint;
    function  HEGetSelectionStartY: Longint;                           
    {Header and Footer}
    function  HEIsHeadFootVisible: Boolean;                            
    function  HEShowHeadFoot(ABool: Boolean): Boolean;
    function  HEGetFootRect: PRect;                                    
    function  HEGetHeadFootHWnd: Longint;
    function  HEGetHeadRect: PRect;                                    
    function  HEGetHeadHWnd: HWnd;                                     
    function  HEGetFootHWnd: HWnd;                                     
    {Get Screen Options}
    function  HEGetTyp: Byte;
    function  HEIsClearDelete: Boolean;
    function  HEIsCrVisible: Boolean;
    function  HEIsFormfeed: Boolean;                                   
    function  HEIsGrayBk: Boolean;                                     
    function  HEIsIgnoreEditable: Boolean;
    function  HEIsInsertMode: Boolean;
    function  HEIsLineBreak: Boolean;                                  
    function  HEIsLineEditable: Boolean;
    function  HEIsLineGhost: Boolean;                                  
    function  HEIsModified: Boolean;
    function  HEIsPercentThumb: Boolean;
    function  HEIsReadOnly: Boolean;                                   
    function  HEIsSpaceVisible: Boolean;
    {Set Screen Options}
    function  HESetCrVisible(ASwitch: Boolean): Boolean;               
    function  HESetSpaceVisible(ASwitch: Boolean): Boolean;
    function  HESetSpacing(ASpacing: Word): Boolean;                   
    function  HEGetSpacing: Word;
    function  HEExtSetRedraw(AScreen, ARepaint, ASave: Boolean): Boolean;
    function  HESetGrayBk(AGray: Boolean): Boolean;
    {Format}
    function  HEPaginate(AHeadHandle, AFootHandle: HWnd): Boolean;      
    function  HESetAlign(AAlign: Word): Boolean;                        
    function  HEFmtLines(ASwitch: Boolean): Boolean;                    
    function  HEFormat: Boolean;                                        
    {Set Options}
    function  HESetAbortDlg: Boolean;                                   
    function  HESetAutoHSize(ASwitch: Boolean): Boolean;
    function  HESetClearDelete(ASwitch: Boolean): Boolean;              
    function  HESetContext(AContext: Word): Boolean;                    
    function  HESetDocInfo(APointer: PSetDocInfo): Boolean;             
    function  HESetEOFChar(AChar: char): Boolean;                       
    function  HESetHScroll(AWindow: HWND): Boolean;                     
    function  HESetIgnoreEditable(ASwitch: Boolean): Boolean;
    function  HESetInsert(ASwitch: Boolean): Boolean;
    function  HESetLineBreak(ASwitch: Boolean): Boolean;
    function  HESetLineeditable(ASwitch: Boolean): Boolean;
    function  HESetLineGhost(ASwitch: Boolean): Boolean;                
    function  HESetModified(ASwitch: Boolean): Boolean;
    function  HESetOffset(ARepaint: Boolean; AX, AY: Word): Boolean;
    function  HESetPageCounter(ACounter: Word): Boolean;
    function  HESetPercentThumb(ASwitch: Boolean): Boolean;
    function  HESetReadOnly(ASwitch: Boolean): Boolean;
    function  HESetScrollPos(APos: longint): Boolean;                   
    function  HESetStyle(AStyle: Word; AColor: TColorRef): Boolean;
    function  HEChangeStyle(AStyle: Word; ASwitch: Boolean): Boolean;
    function  HEInvertStyle(AStyle: Word): Boolean;                     
    function  HESetVScroll(AWindow: HWND): Boolean;                     
    {Font}
    function  HEGetColorLogFont(AIndex: Word; APointer: PColorLogFont): Boolean;       
    function  HEGetScreenColorLogFont(AIndex: Word; APointer: PColorLogFont): Boolean;
    function  HEGetFontHandle(AIndex: Word; AFlag: longint): HFont;                    
    function  HEGetFontIndex: Word;                                                    
    function  HESetColorLogFont(AFlag: Byte; APointer: PColorLogFont): Boolean;        
    function  HEIsScreenFont: Boolean;                                                 
    function  HEIsScreenFontsEnabled: Boolean;
    function  HEEnableScreenFonts(ASwitch: Boolean): Boolean;                          
    {Get Options}
    function  HEGetAlign: Word;
    function  HEGetChar: Word;                                          
    function  HEGetCharLeft: Char;                                      
    function  HEGetCharRight: Char;
    function  HEGetColor: TColorRef;
    function  HEGetContext: Word;
    function  HEGetDocInfo(APointer: PGetDocInfo): Boolean;             
    function  HEGetEofChar: Char;                                       
    function  HEGetLine(AMax: Word; ABuff: PChar): Word;                
    function  HEGetLineCount: Longint;                                  
    function  HEGetPage: Longint;
    function  HEGetPageMax: Word;
    function  HEGetPageCursor: Word;
    function  HEGetRGB(AIndex: Word): Longint;
    function  HEGetScrollPos: longint;                                  
    function  HEGetString(AMax: Word; ABuff: PChar): Word;              
    function  HEGetStyle: Word;
    {Undo}
    function  HECanUndo: Boolean;
    function  HEClearUndoList: Boolean;
    function  HEGetUndoDepth: Word;                                     
    function  HEQueryUndoType(AIndex: Word): Longint;
    function  HESetUndoDepth(ADepth: Word): Boolean;
    function  HEUndo: Boolean;                                          
    {Preview}
    function  HEIsPreview: Boolean;
    function  HEPreview: Boolean;
    {Table}
    function  HEAdjustTable(AFlag: Word): Boolean;
    function  HECloseActiveTable: Boolean;                              
    function  HEDeleteTableRow(ARow: Longint): Boolean;                 
    function  HEInsertTable(ACol, ARow: Word): HWnd;                    
    function  HEInsertTableRow(ARow: Word): Boolean;                    
    function  HEIsTableActive: HWnd;                                    
    function  HETableActivate: HWnd;                                    
    {Clipboard}
    function  HECanCopy: Boolean;                                       
    function  HECanPaste: Boolean;                                      
    function  HECopyData: LongInt;
    function  HEPasteData(AHandle: THandle): Boolean;
    function  HECopyRTF: THandle;
    function  HEPasteRTF(AHandle: THandle): Boolean;                    
    {And More}
    function  HEGetNotifyWParam: Word;
    function  HEGetNotifyLParam: Longint;
    function  HELineLength(ARow: Longint): Longint;
    function  HENew: Boolean;
    function  HEQuickreplace(ALng: Word; AData: pointer): Boolean;
    function  HESearch(AFlags: Word; AText: PChar): Word;
    function  HEStatusLine(AWindow: HWnd; AStatusLinePtr:PStatusLine): Boolean;
    function  HEWriteProfile: Boolean;                                          
    function  HECountToken(AMode: Word; AText: PChar): Longint;
    function  HEFormFeed: Boolean;
    function  HERedraw: Boolean;                                                

    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function  CanClose: Boolean; virtual;
    procedure Resize;
    {File}
    procedure FileNew;
    procedure FileOpen;
    procedure FileInsert;
    procedure FileLoad(AFileFormat: word; AFileName: TFileName);
    procedure FileSave;
    procedure FileSaveAs;
    procedure FileInfo;
    procedure FilePrint;
    procedure FilePrintSetup;
    procedure FileExit;
    {Edit}
    procedure EditUndo;
    procedure EditCopy;
    procedure EditPaste;
    procedure EditCut;
    procedure EditClear;
    procedure EditSelect;
    {Font}
    procedure Standard;
    procedure Bold;
    procedure Underline;
    procedure Italic;
    procedure StrikeOut;
    procedure SubScript;
    procedure SuperScript;
    procedure FlushLeft;
    procedure FlushRight;
    procedure FlushCenter;
    procedure FlushJustify;
    procedure SpacingOne;
    procedure SpacingOneHalf;
    procedure SpacingDouble;
    procedure Color;
    procedure NewFont;
    procedure ScreenFonts;
    {Search}
    procedure Find;
    procedure Replace;
    procedure SearchCont;
    function  SearchContinue(ADisplayError: Bool; AIgnoreInit: Bool): Word;   virtual;
    function  Search(AFlags: Word; ABuff: PChar; ADisplayError: Bool): Word;  virtual;
    procedure SearchResult(AResult: Word; ADisplayError: Bool);               virtual;
    {Options}
    procedure DocFormat;
    procedure Paragraph;
    procedure BackColor;
    procedure LineBreak;
    procedure PictureFrame;
    procedure CRVisible;
    procedure TabVisible;
    procedure SpaceVisible;
    procedure PFVisible;
    procedure PFFunction;
    procedure PFGhost;
    procedure LineGhost;
    procedure UnitCM;
    procedure UnitIN;
    procedure SaveOptions;
    {Extras}
    procedure HeadFoot;
    procedure Preview;
    procedure InsertPicture;
    procedure InsertTable;
    procedure InsertTableRow;
    procedure DeleteTableRow;
    procedure Format;
    procedure FormFeed;
    procedure Paginate;

    property Modified: Boolean read HEIsModified write HESetModified;
    property Redraw: Boolean read FRedraw write SetRedraw default True;
  published
    property Align;
    property BkColor: TColor read GetBkColor write SetBkColor;
    property DblClickSelectWord: Boolean read FDblClickSelectWord write FDblClickSelectWord default True;
    property FileName: TFileName read GetFileName write SetFileName;
    property GrayBk: Boolean read GetGrayBk write SetGrayBk;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnInitNew: TNotifyEvent read FOnInitNew write FOnInitNew;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnPictureSelect: TNotifyEvent read FOnPictureSelect write FOnPictureSelect;
    property ShowControls: TShowControls read GetShowControls write SetShowControls;
    property ShowHeadFoot: Boolean read HEIsHeadFootVisible write HEShowHeadFoot;
    property StatusFont: TFont read FStatusFont write SetStatusFont;
    property TabOrder;
    property TabStop;
    property TextBorderStyle: TBorderStyle read FTextBorderStyle write SetTextBorderStyle default bsSingle;
    property TextSelectable: Boolean read HEIsTextSelectable write SetTextSelectable;
    property UnitType: TUnitType read GetUnitType write SetUnitType;
  end;

implementation

uses DsgnIntf;

{======================================================================}

function MakeWord(ALo, AHi: Byte): Word;
var W : Word;
begin
    W := AHi;
    MakeWord := W shl 8 + ALo;
end;

{======================================================================}

{THighEdit}

constructor THighEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls];
  Width := 100;
  Height := 100;
  Align := alNone;

  FBkBrush := TBrush.Create;
  FDblClickSelectWord := True;
  FDoSaveAs := True;
  FPrintHead := True;
  FPrintFoot := True;
  FRedraw := True;
  FStatusFont := TFont.Create;
  with FStatusFont do begin
    Height := 16;
    Pitch := fpVariable;
    Name := 'Arial';
    OnChange := ChangeStatusFont;
  end;
  FTextBorderStyle := bsSingle;
  FUntitled := True;
  FUseTab := True;
  FUseRuler := True;
  FUseStatus := True;
  FUseIcon := True;

  GetMem(FFileBoxParamPtr, SizeOf(TFileBoxParam));
  fillchar(FFileBoxParamPtr^, SizeOf(TFileBoxParam), 0);
  FFileBoxParamPtr^.wFileType := FileFormat_HighEdit;

  GetMem(FPictureFileBoxParamPtr, SizeOf(TFileBoxParam));
  fillchar(FPictureFileBoxParamPtr^, SizeOf(TFileBoxParam), 0);
  FPictureFileBoxParamPtr^.wFileType := FileFormat_Bitmap;

  GetMem(FColorBoxParamPtr, SizeOf(TColorBoxParam));
  fillchar(FColorBoxParamPtr^, SizeOf(TColorBoxParam), 0);

  GetMem(FBackColorBoxParamPtr, SizeOf(TColorBoxParam));
  fillchar(FBackColorBoxParamPtr^, SizeOf(TColorBoxParam), 0);

  GetMem(FFontBoxParamPtr, SizeOf(TFontBoxParam));
  fillchar(FFontBoxParamPtr^, SizeOf(TFontBoxParam), 0);

  GetMem(FSearchBoxParamPtr, SizeOf(TSearchBoxParam));
  fillchar(FSearchBoxParamPtr^, SizeOf(TSearchBoxParam), 0);

  GetMem(FParagraphBoxParamPtr, SizeOf(TParagraphBoxParam));
  fillchar(FParagraphBoxParamPtr^, SizeOf(TParagraphBoxParam), 0);

  GetMem(FFormatDocBoxParamPtr, SizeOf(TFormatDocBoxParam));
  fillchar(FFormatDocBoxParamPtr^, SizeOf(TFormatDocBoxParam), 0);

  GetMem(FDocInfoBoxParamPtr, SizeOf(TDocInfoBoxParam));
  fillchar(FDocInfoBoxParamPtr^, SizeOf(TDocInfoBoxParam), 0);

  GetMem(FPrintBoxParamPtr, SizeOf(TPrintBoxParam));
  fillchar(FPrintBoxParamPtr^, SizeOf(TPrintBoxParam), 0);
end;

{----------------------------------------------------------------------}

destructor THighEdit.Destroy;
begin
  FStatusFont.Free;
  FBkBrush.Free;
  FreeMem(FFileBoxParamPtr, SizeOf(TFileBoxParam));
  FreeMem(FPictureFileBoxParamPtr, SizeOf(TFileBoxParam));
  FreeMem(FColorBoxParamPtr, SizeOf(TColorBoxParam));
  FreeMem(FBackColorBoxParamPtr, SizeOf(TColorBoxParam));
  FreeMem(FFontBoxParamPtr, SizeOf(TFontBoxParam));
  FreeMem(FSearchBoxParamPtr, SizeOf(TSearchBoxParam));
  FreeMem(FParagraphBoxParamPtr, SizeOf(TParagraphBoxParam));
  FreeMem(FFormatDocBoxParamPtr, SizeOf(TFormatDocBoxParam));
  FreeMem(FDocInfoBoxParamPtr, SizeOf(TDocInfoBoxParam));
  FreeMem(FPrintBoxParamPtr, SizeOf(TPrintBoxParam));
  Inherited Destroy;
end;

{----------------------------------------------------------------------}

procedure THighEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
end;

{----------------------------------------------------------------------}

procedure THighEdit.CreateWnd;
begin
  inherited CreateWnd;
  if FUseTab then CreateTabulator;
  if FUseStatus then CreateStatusLine;
  if FUseRuler then CreateRuler;
  if FUseIcon then CreateIconBar;
  CreateHighEdit;
  WinProcs.SetFocus (FHighEditHandle);

  NewWindowTitle;
  Resize;
end;

{----------------------------------------------------------------------}

procedure THighEdit.CreateHighEdit;
const
  BorderStyle: array[TBorderStyle] of LongInt = (0, WS_BORDER);
var
   wsD: Longint;
begin
  if FHighEditHandle <> 0 then DestroyWindow(FHighEditHandle);
  if csDesigning in ComponentState
  then wsD := WS_DISABLED
  else wsD := 0;
  FHighEditHandle := WinProcs.CreateWindow ( class_HighEdit, '',
                     ws_DefHighEdit or WS_HSCROLL or WS_VSCROLL or HS_NOTIFY or
                     BorderStyle[FTextBorderStyle] or hs_HeadFoot or hs_Undo or wsD,
                     0, 0, 0, 0, Handle, id_HighEdit, HInstance, nil);
  if (FHighEditHandle<>0) then HESetOffset(false, 15, 5);
  FSearchBoxParamPtr^.wSearchFlags   := search_Init;
  FSearchBoxParamPtr^.hwndEdit      := FHighEditHandle;
  FFormatDocBoxParamPtr^.hwndEdit   := FHighEditHandle;
  FDocInfoBoxParamPtr^.hwndEdit     := FHighEditHandle;
  FParagraphBoxParamPtr^.hwndEdit   := FHighEditHandle;

  if HEIsScreenFontsEnabled then begin
    FFontBoxParamPtr^.bScreenFontsEnabled := true;
    IBMEnableScreenFonts(true);
  end
  else FFontBoxParamPtr^.bScreenFontsEnabled := False;
end;

{----------------------------------------------------------------------}

procedure THighEdit.CreateIconBar;
var
   wsD: Longint;
begin
  if FIconHandle <> 0
  then DestroyWindow(FIconHandle);
  if csDesigning in ComponentState
  then wsD := WS_DISABLED
  else wsD := 0;
  FIconHandle := WinProcs.CreateWindow ( class_IconBar, '', ws_DefIconBar or wsD,
  	         0, 0, 0, 0, Handle, id_Icon, HInstance, nil);
end;

{----------------------------------------------------------------------}

procedure THighEdit.CreateRuler;
var
   wsD: Longint;
begin
  if FRulerHandle <> 0 then DestroyWindow(FRulerHandle);
  if csDesigning in ComponentState
  then wsD := WS_DISABLED
  else wsD := 0;
  FRulerHandle := WinProcs.CreateWindow ( class_Ruler, '', ws_DefRuler or wsD,
                  0, 0, 0, 0, Handle, id_Ruler, HInstance, nil);
  if (FRulerHandle<>0) then SendMessage(FRulerHandle, rl_SetOffset, 0, -15);
end;

{----------------------------------------------------------------------}

procedure THighEdit.CreateStatusLine;
const
  pSpalte: PChar = 'Spalte';
  pZeile : PChar = 'Zeile';
  pSeite : PChar = 'Seite';
var
   wsD: Longint;
begin
  if FStatusHandle <> 0 then DestroyWindow(FStatusHandle);
  if csDesigning in ComponentState
  then wsD := WS_DISABLED
  else wsD := 0;
  FStatusHandle := WinProcs.CreateWindow( class_Status, '', ws_DefStatus or ws_Border or wsD,
                    0, 0, 0, 0, Handle, id_Status, HInstance, nil);
  if (FStatusHandle<>0) then begin
    SendMessage(FStatusHandle, wm_SetFont, FStatusFont.Handle, 0);
    SendMessage(FStatusHandle, sl_SetItem, 60, 1);
    SendMessage(FStatusHandle, sl_SetItem, 50, 0);
    SendMessage(FStatusHandle, sl_SetItem, 60, 1);
    SendMessage(FStatusHandle, sl_SetItem, 50, 0);
    SendMessage(FStatusHandle, sl_SetItem, 80, 1);
    SendMessage(FStatusHandle, sl_SetItem, 50, 0);
    SendMessage(FStatusHandle, sl_SetItemText, 1, Longint (pSpalte));
    SendMessage(FStatusHandle, sl_SetItemText, 3, Longint (pZeile));
    SendMessage(FStatusHandle, sl_SetItemText, 5, Longint (pSeite));
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.CreateTabulator;
var
   wsD: Longint;
begin
  if FTabHandle <> 0 then DestroyWindow(FTabHandle);
  if csDesigning in ComponentState
  then wsD := WS_DISABLED
  else wsD := 0;
  FTabHandle := WinProcs.CreateWindow( class_Tab, '', ws_DefTab or wsD,
                0, 0, 0, 0, Handle, id_Tab, HInstance, nil);
  if (FTabHandle<>0) then SendMessage(FTabHandle, tm_SetOffset, 0, 15);
end;

{----------------------------------------------------------------------}

procedure THighEdit.Resize;
var
  iStatusHeight       : Integer;
  y                   : Word;
  rc                  : TRect;
begin
  WinProcs.GetClientRect(Handle, rc);
  IBMHideDropDown;
  y := 0;
  iStatusHeight := 0;
  if FStatusHandle<>0 then iStatusHeight := SendMessage(FStatusHandle, sl_GetHeight, 0, 0);
  if FIconHandle <> 0 then begin
    MoveWindow (FIconHandle, 0, 0, rc.Right-rc.Left, 32, true);
    y := y + 32;
  end;
  if FRulerHandle <> 0 then begin
    MoveWindow (FRulerHandle, 0, y, rc.Right-rc.Left, 16, true);
    y := y + 16;
  end;
  if FTabHandle <> 0 then begin
    MoveWindow (FTabHandle, 0, y, rc.Right-rc.Left, 16, true);
    y := y + 16;
  end;
  if FHighEditHandle <> 0 then begin
    MoveWindow (FHighEditHandle, -1, y, rc.Right-rc.Left+2, rc.Bottom-rc.Top - y - iStatusHeight+2, true);
    y := rc.Bottom-rc.Top - iStatusHeight+1;
  end;
  if FStatusHandle <> 0 then MoveWindow (FStatusHandle, -1, y, rc.Right-rc.Left+2, rc.Bottom-rc.Top - y+1, true);
end;

(*** File ***************************************************************************************)


procedure THighEdit.FileExit;
begin
  SendMessage(Handle, wm_Close, 0, 0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.FileInfo;
begin
  DocInfoDialogBox(Handle, FDocInfoBoxParamPtr);
end;

{----------------------------------------------------------------------}

procedure THighEdit.FileInsert;
var
  TmpFileBoxParamPtr  : TFileBoxParam;
begin
  TmpFileBoxParamPtr := FFileBoxParamPtr^;
  if FileAppendDialogBox(Handle, FFileBoxParamPtr) then HEAppend;
  FFileBoxParamPtr^ := TmpFileBoxParamPtr;
end;

{----------------------------------------------------------------------}

procedure THighEdit.FileLoad(AFileFormat: word; AFileName: TFileName);
var Name: Array[0..79] of Char;
begin
  StrPCopy(Name, AFileName);
  if CanClose then begin
    if HELoadExt(AFileFormat, Name) then begin
      FUntitled := false;
      Modified := FALSE;
      NewWindowTitle;
    end;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.FileNew;
begin
  if CanClose then begin
    HENew;
    FUntitled := true;
    NewWindowTitle;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.FileOpen;
begin
  if CanClose then begin
    FFileBoxParamPtr^.hfontThin := FStatusFont.Handle;
    if FileOpenDialogBox(Handle, FFileBoxParamPtr) then begin
      if (HELoad) then begin
        FUntitled := false;
        NewWindowTitle;
      end;
    end;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.FilePrint;
var
  szName: array[0..29] of Char;
begin
  if PrintDialogBox(Handle, FPrintBoxParamPtr) then begin
    if FUntitled
    then LoadString (hInstance, ids_Untitled, szName, SizeOf (szName))
    else begin
      StrCopy(szName, FFileBoxParamPtr^.szFileName);
      StrUpper(szName);
    end;
    FPrintBoxParamPtr^.pdp.pDocTitle := szName;
    HEPrintDocument(print_FullDocumentAbortDlg, @FPrintBoxParamPtr^.pdp);
    DeleteDC(FPrintBoxParamPtr^.pdp.hdc);
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.FilePrintSetup;
begin
  PrintSetupDialogBox(Application.MainForm.Handle, FPrintBoxParamPtr);
end;

{----------------------------------------------------------------------}

procedure THighEdit.FileSave;
begin
  if not FUntitled then begin
    FDoSaveAs := False;
    if HESave then begin
      FUntitled := False;
      NewWindowTitle;
    end;
  end
  else begin
    if FileSaveDialogBox(Handle, FFileBoxParamPtr) then begin
      FDoSaveAs := true;
      if HESave then begin
        FUntitled := False;
        NewWindowTitle;
      end;
    end;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.FileSaveAs;
begin
  if FileSaveDialogBox(Handle, FFileBoxParamPtr) then begin
    FDoSaveAs := true;
    if HESave then begin
      FUntitled := False;
      NewWindowTitle;
    end;
  end;
end;


(*** Edit ***************************************************************************************)


procedure THighEdit.EditClear;
begin
  SendMessage(FHighEditHandle, wm_Clear, 0, 0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.EditCopy;
begin
  SendMessage(FHighEditHandle, wm_Copy, 0, 0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.EditCut;
begin
  SendMessage(FHighEditHandle, wm_Cut, 0, 0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.EditPaste;
begin
  SendMessage(FHighEditHandle, wm_Paste, 0, 0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.EditSelect;
begin
  SendMessage(FHighEditHandle, he_SelectAll, 0, 0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.EditUndo;
begin
  HEUndo;
end;


(*** Font ***************************************************************************************)


procedure THighEdit.Bold;
begin
  HEInvertStyle(attr_Bold);
end;

{----------------------------------------------------------------------}

procedure THighEdit.Color;
var
  DlgColor: TColorDialog;
begin
  try
    DlgColor := TColorDialog.Create(self);
    DlgColor.Color := SendMessage(FHighEditHandle, he_GetRGB, Word (-1), 0);
    if DlgColor.Execute
    then SendMessage(FHighEditHandle, he_SetStyle, current_Attribute, DlgColor.Color);
  finally
    DlgColor.Free;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.FlushCenter;
begin
  HESetAlign(align_Center);
end;

{----------------------------------------------------------------------}

procedure THighEdit.FlushJustify;
begin
  HESetAlign(align_Justify);
end;

{----------------------------------------------------------------------}

procedure THighEdit.FlushLeft;
begin
  HESetAlign(align_Left);
end;

{----------------------------------------------------------------------}

procedure THighEdit.FlushRight;
begin
  HESetAlign(align_Right);
end;

{----------------------------------------------------------------------}

procedure THighEdit.Italic;
begin
  HEInvertStyle(attr_Italic);
end;

{----------------------------------------------------------------------}

procedure THighEdit.NewFont;
begin
  FFontBoxParamPtr^.bIsScreenFont := HEIsScreenFont;
  if FFontBoxParamPtr^.bIsScreenFont
  then HEGetScreenColorLogFont(Word(-1), @FFontBoxParamPtr^.clf)
  else HEGetColorLogFont(Word(-1), @FFontBoxParamPtr^.clf);
  FFontBoxParamPtr^.bScreenFontsEnabled := HEIsScreenFontsEnabled;
  if FontDialogBox(Handle, FFontBoxParamPtr) then begin
    HEEnableScreenFonts(FFontBoxParamPtr^.bScreenFontsEnabled);
    IBMEnableScreenFonts(FFontBoxParamPtr^.bScreenFontsEnabled);
    if FFontBoxParamPtr^.bIsScreenFont
    then HESetColorLogFont(font_Display or font_Set_All, @FFontBoxParamPtr^.clf)
    else HESetColorLogFont(font_Printer or font_Set_All, @FFontBoxParamPtr^.clf);
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.ScreenFonts;
var
  bSet: Bool;
begin
  bSet := HEIsScreenFontsEnabled;
  HEEnableScreenFonts(not bSet);
  IBMEnableScreenFonts(not bSet);
end;

{----------------------------------------------------------------------}

procedure THighEdit.SpacingDouble;
begin
  HESetSpacing(spacing_Double);
end;

{----------------------------------------------------------------------}

procedure THighEdit.SpacingOne;
begin
  HESetSpacing(spacing_One);
end;

{----------------------------------------------------------------------}

procedure THighEdit.SpacingOneHalf;
begin
  HESetSpacing(spacing_OneHalf);
end;

{----------------------------------------------------------------------}

procedure THighEdit.Standard;
begin
  HESetStyle(attr_Standart, current_Color);
end;

{----------------------------------------------------------------------}

procedure THighEdit.StrikeOut;
begin
  HEInvertStyle(attr_Strikeout);
end;

{----------------------------------------------------------------------}

procedure THighEdit.SubScript;
begin
  HEInvertStyle(attr_Subscript);
end;

{----------------------------------------------------------------------}

procedure THighEdit.SuperScript;
begin
  HEInvertStyle(attr_Superscript);
end;

{----------------------------------------------------------------------}

procedure THighEdit.Underline;
begin
  HEInvertStyle(attr_Underline);
end;


(*** Search *************************************************************************************)


function THighEdit.Search(AFlags: Word; ABuff: PChar; ADisplayError: Bool): Word;
begin
  Result := HESearch(AFlags, ABuff);
  StrLCopy(FSearchBoxParamPtr^.szSearchText, ABuff, SizeOf(FSearchBoxParamPtr^.szSearchText)-1);
  FSearchBoxParamPtr^.szSearchText[SizeOf(FSearchBoxParamPtr^.szSearchText)-1] := #0;
  FSearchBoxParamPtr^.wSearchFlags := AFlags or search_Init;
  SearchResult(Result, ADisplayError);
end;

{----------------------------------------------------------------------}

function THighEdit.SearchContinue;
begin
  Result := 0;
  if ((not AIgnoreInit) and Bool(FSearchBoxParamPtr^.wSearchFlags and search_Init))
  then Result := search_Error
  else begin
    Result := HESearch(FSearchBoxParamPtr^.wSearchFlags, FSearchBoxParamPtr^.szSearchText);
    SearchResult(Result, ADisplayError);
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.Find;
begin
  if SearchDialogBox(Handle, FSearchBoxParamPtr)
  then Search(FSearchBoxParamPtr^.wSearchFlags, FSearchBoxParamPtr^.szSearchText, true);
end;

{----------------------------------------------------------------------}

procedure THighEdit.Replace;
var
  Result: Word;
  lYPos : Longint;
  wXPos : Word;
begin
  Result := ReplaceDialogBox(Handle, FSearchBoxParamPtr);
  case Result of
  1:begin { Ersetzen mit Rckfrage }
      if Search(FSearchBoxParamPtr^.wSearchFlags, FSearchBoxParamPtr^.szSearchText, true)=search_Found
      then QueryReplaceDialogBox(Handle, FSearchBoxParamPtr);
    end;
  2:begin { Alles ersetzen }
      Redraw := False;
      if Search( FSearchBoxParamPtr^.wSearchFlags,
                 FSearchBoxParamPtr^.szSearchText, true) = search_Found then begin
        repeat
          lYPos := HEGetCursorX;
          wXPos := HEGetCursorY;
          HEReplaceSel(FSearchBoxParamPtr^.szReplaceText);
          FSearchBoxParamPtr^.wSearchFlags := FSearchBoxParamPtr^.wSearchFlags or search_Cursor or search_Init;
          if (FSearchBoxParamPtr^.wSearchFlags and search_Back<>0)
          then HESetCursor(wXPos, lYPos);
        until (SearchContinue(False, true)<>search_Found);
        FSearchBoxParamPtr^.wSearchFlags := FSearchBoxParamPtr^.wInternalSearchFlags;
        FSearchBoxParamPtr^.wSearchFlags := FSearchBoxParamPtr^.wSearchFlags or search_Init;
      end;
      Redraw := True;
    end;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.SearchCont;
begin
  SearchContinue(true, False);
end;

{----------------------------------------------------------------------}

procedure THighEdit.SearchResult(AResult: Word; ADisplayError: Bool);
begin
  case AResult of
    search_Error:
    begin
      FSearchBoxParamPtr^.wSearchFlags := FSearchBoxParamPtr^.wSearchFlags or search_Init;
      if ADisplayError then HEError(ids_SearchError, mtWarning, [mbOK]);
    end;
    search_NotFound:
    begin
      FSearchBoxParamPtr^.wSearchFlags := FSearchBoxParamPtr^.wSearchFlags or search_Init;
      if ADisplayError then HEError(ids_SearchNotFound, mtWarning, [mbOK]);
    end;
    search_Found:
    begin
      FSearchBoxParamPtr^.wSearchFlags := FSearchBoxParamPtr^.wSearchFlags and not search_Init;
    end;
  end;
end;


(*** Options ************************************************************************************)


procedure THighEdit.BackColor;
begin
  FBackColorBoxParamPtr^.crPF := SendMessage(FHighEditHandle, he_GetPFBkColor, 0, 0);
  if BackColorDialogBox(Handle, FBackColorBoxParamPtr) then begin
    FBkBrush.Color :=  FBackColorBoxParamPtr^.crBk;
    SendMessage(FHighEditHandle, he_SetPFBkColor, 0, FBackColorBoxParamPtr^.crPF);
    InvalidateRect (FHighEditHandle, nil, true);
    if HEIsHeadFootVisible then begin
      InvalidateRect(HEGetHeadHwnd, nil, true);
      InvalidateRect(HEGetFootHwnd, nil, true);
    end;
    UpdateWindow (FHighEditHandle);
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.CRVisible;
begin
  HESetCRVisible(not HEIsCRVisible);
end;

{----------------------------------------------------------------------}

procedure THighEdit.DocFormat;
begin
  FormatDocDialogBox(Application.MainForm.Handle, FFormatDocBoxParamPtr);
end;

{----------------------------------------------------------------------}

procedure THighEdit.LineBreak;
begin
  HESetLineBreak(not HEIsLineBreak);
end;

{----------------------------------------------------------------------}

procedure THighEdit.LineGhost;
begin
  HESetLineGhost(not HEIsLineGhost);
end;

{----------------------------------------------------------------------}

procedure THighEdit.Paragraph;
begin
  ParagraphDialogBox(Handle, FParagraphBoxParamPtr);
end;

{----------------------------------------------------------------------}

procedure THighEdit.PFFunction;
begin
  HESetPFFunctionVisible(not HEIsPFFunctionVisible);
end;

{----------------------------------------------------------------------}

procedure THighEdit.PFGhost;
begin
  HESetPFGhost(not HEIsPFGhost);
end;

{----------------------------------------------------------------------}

procedure THighEdit.PFVisible;
begin
  HESetPFBkVisible(not HEIsPFBkVisible);
end;

{----------------------------------------------------------------------}

procedure THighEdit.PictureFrame;
begin
  HESetPictureFrame(not HEIsPictureFrame);
end;

{----------------------------------------------------------------------}

procedure THighEdit.SaveOptions;
begin
  HEWriteProfile;
end;

{----------------------------------------------------------------------}

procedure THighEdit.SpaceVisible;
begin
  HESetSpaceVisible(not HEIsSpaceVisible);
end;

{----------------------------------------------------------------------}

procedure THighEdit.TabVisible;
begin
  HESetTabVisible(not HEIsTabVisible);
end;

{----------------------------------------------------------------------}

procedure THighEdit.UnitCM;
begin
  if (FRulerHandle<>0) then SendMessage(FRulerHandle, rl_SetUnit, unit_CM, 0);
  if (FTabHandle<>0) then SendMessage(FTabHandle, tm_SetUnit, unit_CM, 0);
  HESetUnit(unit_CM);
end;

{----------------------------------------------------------------------}

procedure THighEdit.UnitIN;
begin
  if (FRulerHandle<>0) then SendMessage(FRulerHandle, rl_SetUnit, unit_IN, 0);
  if (FTabHandle<>0) then SendMessage(FTabHandle, tm_SetUnit, unit_IN, 0);
  HESetUnit(unit_IN);
end;

(*** Extras *************************************************************************************)


procedure THighEdit.DeleteTableRow;
begin
  HEDeleteTableRow(0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.Format;
begin
  HEFormat;
end;

{----------------------------------------------------------------------}

procedure THighEdit.FormFeed;
begin
  HEFormFeed;
end;

{----------------------------------------------------------------------}

procedure THighEdit.HeadFoot;
begin
  HEShowHeadFoot(not HEIsHeadFootVisible);
end;

{----------------------------------------------------------------------}

procedure THighEdit.InsertPicture;
begin
  if PictureOpenDialogBox(Handle, FPictureFileBoxParamPtr)
  then HELoadPicture;
end;

{----------------------------------------------------------------------}

procedure THighEdit.InsertTable;
var
  DlgTabInsert: TTabInsertDlg;
  Columns, Rows       : Word;
  Return              : Integer;
begin
  DlgTabInsert := TTabInsertDlg.Create(Application);
  DlgTabInsert.EdtRows.Text := '2';
  DlgTabInsert.EdtCols.Text := '2';
  Return := DlgTabInsert.ShowModal;
  Rows :=  StrToInt(DlgTabInsert.EdtRows.Text);
  Columns := StrToInt(DlgTabInsert.EdtCols.Text);
  DlgTabInsert.Free;
  if Return = mrOk then HEInsertTable(Columns, Rows);
end;

{----------------------------------------------------------------------}

procedure THighEdit.InsertTableRow;
begin
  HEInsertTableRow(0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.Paginate;
begin
  HEPaginate(0, 0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.Preview;
begin
  HEPreview;
end;


(*** More ***************************************************************************************)


function THighEdit.CanClose: Boolean;
var
  iReturn  : Integer;
  FName: String;
begin
  if not HECloseActivePF(false) then begin
    CanClose := False;
    Exit;
  end;
  if not HEIsModified then begin
    CanClose := true;
    Exit;
  end;
  if FUntitled
  then FName := LoadStr(ids_Untitled)
  else FName := UpperCase(StrPas(FFileBoxParamPtr^.szFileName));
  iReturn := MessageDlg(FmtLoadStr(ids_CloseSave, [FName]), mtConfirmation, mbYesNoCancel, 0);
  case iReturn of
  idYes:
    begin
      if not FUntitled then begin
        if HESave then begin
          CanClose := true;
          Exit;
        end;
      end
      else if FileSaveDialogBox(Handle, FFileBoxParamPtr) then if HESave then begin
        FUntitled := False;
        NewWindowTitle;
        CanClose := true;
        Exit;
      end;
    end;
  idNo:
    begin
      CanClose := true;
      Exit;
    end;
  end;
  CanClose := False;
end;

{----------------------------------------------------------------------}

procedure THighEdit.NewWindowTitle;
begin
  if FUntitled
  then Caption := LoadStr(ids_Title) + ' - ' + LoadStr(ids_Untitled)
  else Caption := LoadStr(ids_Title) + ' - ' + UpperCase(StrPas(FFileBoxParamPtr^.szFileName));
end;

{----------------------------------------------------------------------}

procedure THighEdit.SetRedraw(Value: Boolean);
begin
  FRedraw := Value;
  SendMessage(FHighEditHandle, wm_SetRedraw, Word(Value), 0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.WMCommand(var Msg: TWMCommand);
begin
  if (Msg.Ctl<>0) then begin
    case Msg.ItemID of
      id_HighEdit: begin
        if (FHighEditHandle=0) then FHighEditHandle := Msg.Ctl;
        case Msg.NotifyCode of
        hef_NoMemory:        HEError(ids_NoMemory,      mtError,   [mbOK]);
        hef_LowMemory:       HEError(ids_LowMemory,     mtWarning, [mbOK]);
        hef_NoSave:          HEError(ids_NoSave,        mtWarning, [mbOK]);
        hef_NoLoad:          HEError(ids_NoLoad,        mtWarning, [mbOK]);
        hef_NoStart:         HEError(ids_NoStart,       mtError,   [mbOK]);
        hef_NoBlockMark:     HEError(ids_NoBlockMark,   mtWarning, [mbOK]);
        hef_NoTmpOpen:       HEError(ids_NoTmpOpen,     mtError,   [mbOK]);
        hef_NoTmpClose:      HEError(ids_NoTmpClose,    mtWarning, [mbOK]);
        hef_NoHEFile:        HEError(ids_NoHEFile,      mtWarning, [mbOK]);
        hef_NoTmpSpace:      HEError(ids_NoTmpSpace,    mtError,   [mbOK]);
        hef_NoRTFFile:       HEError(ids_NoRTFFile,     mtWarning, [mbOK]);
        hef_WrongVersion:    HEError(ids_WrongVersion,  mtWarning, [mbOK]);
        hef_NoFont:          HEError(ids_NoFont,        mtWarning, [mbOK]);
        hef_NoPrinter:       HEError(ids_NoPrinter,     mtWarning, [mbOK]);
        hef_NoWriteAccess:   HEError(ids_NoWriteAccess, mtWarning, [mbOK]);
        hem_Align:           HEMAlign(TMessage(Msg));
        hem_Char:            HEMChar(TMessage(Msg));
        hem_Click:           HEMClick(TMessage(Msg));
        hem_CursorPos:       HEMCursorPos(TMessage(Msg));
        hem_DblClk:          HEMDblClk(TMessage(Msg));
        hem_EndPF:           HEMEndPF(TMessage(Msg));
        hem_FileExists:      HEMFileExists(TMessage(Msg));
        hem_Font:            HEMFont(TMessage(Msg));
        hem_FontSize:        HEMFontSize(TMessage(Msg));
        hem_FormatPF:        HEMFormatPF(TMessage(Msg));
        hem_HScroll:         HEMHScroll(TMessage(Msg));
        hem_InitNew:         HEMInitNew(TMessage(Msg));
        hem_Insert:          HEMInsert(TMessage(Msg));
        hem_InsertPF:        HEMInsertPF(TMessage(Msg));
        hem_KeyDown:         HEMKeyDown(TMessage(Msg));
        hem_KeyUp:           HEMKeyUp(TMessage(Msg));
        hem_KillFocus:       HEMKillFocus(TMessage(Msg));
        hem_LButtonDown:     HEMLButtonDown(TMessage(Msg));
        hem_LButtonUp:       HEMLButtonUp(TMessage(Msg));
        hem_MessageText:     HEMMessageText(TMessage(Msg));
        hem_MButtonDown:     HEMMButtonDown(TMessage(Msg));
        hem_MButtonUp:       HEMMButtonUp(TMessage(Msg));
        hem_MouseMove:       HEMMouseMove(TMessage(Msg));
        hem_NewPageFragment: HEMNewPageFragment(TMessage(Msg));
        hem_PrintFoot:       HEMPrintFoot(TMessage(Msg));
        hem_PrintHead:       HEMPrintHead(TMessage(Msg));
        hem_RButtonDown:     HEMRButtonDown(TMessage(Msg));
        hem_RButtonUp:       HEMRButtonUp(TMessage(Msg));
        hem_SelectPF:        HEMSelectPF(TMessage(Msg));
        hem_SelectPicture:   HEMSelectPicture(TMessage(Msg));
        hem_SetFocus:        HEMSetFocus(TMessage(Msg));
        hem_SetHScrollPos:   HEMSetHScrollPos(TMessage(Msg));
        hem_SetHScrollRange: HEMSetHScrollRange(TMessage(Msg));
        hem_SetVScrollPos:   HEMSetVScrollPos(TMessage(Msg));
        hem_SetVScrollRange: HEMSetVScrollRange(TMessage(Msg));
        hem_SizePF:          HEMSizePF(TMessage(Msg));
        hem_Spacing:         HEMSpacing(TMessage(Msg));
        hem_Style:           HEMStyle(TMessage(Msg));
        hem_TabModify:       HEMTabModify(TMessage(Msg));
        hem_TabWidth:        HEMTabWidth(TMessage(Msg));
        hem_VScroll:         HEMVScroll(TMessage(Msg));
        end;
      end;
      id_Tab: begin
        if (FTabHandle=0) then FTabHandle := Msg.Ctl;
        case HiByte(Msg.NotifyCode) of
        tn_Int_Insert: TNIntInsert(TMessage(Msg));
        tn_Int_Update: TNIntUpdate(TMessage(Msg));
        tn_Int_Delete: TNIntDelete(TMessage(Msg));
        end;
      end;
      id_Ruler:
        case Msg.NotifyCode of
        rn_LButtonDblClk: RNLButtonDblClk(TMessage(Msg));
        rn_RButtonDblClk: RNRButtonDblClk(TMessage(Msg));
        end;
      id_Icon: begin
        if (FIconHandle=0) then FIconHandle := Msg.Ctl;
        case Msg.NotifyCode of
        ibn_Align:     IBNAlign(TMessage(Msg));
        ibn_Color:     IBNColor(TMessage(Msg));
        ibn_DblClk:    IBNDblClk(TMessage(Msg));
        ibn_Display:   IBNDisplay(TMessage(Msg));
        ibn_FontName:  IBNFontName(TMessage(Msg));
        ibn_FontSize:  IBNFontSize(TMessage(Msg));
        ibn_Spacing:   IBNSpacing(TMessage(Msg));
        ibn_Style:     IBNStyle(TMessage(Msg));
        ibn_Tab:       IBNTab(TMessage(Msg));
        end;
        WinProcs.SetFocus (FHighEditHandle)
      end;
      id_Status: IDStatus(TMessage(Msg));
    end;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.WMCtlColor(var Msg: TMessage);
begin
  Msg.result := FBkBrush.Handle;
end;

{----------------------------------------------------------------------}

procedure THighEdit.WMEndSession(var Msg: TMessage);
begin
  if (FHighEditHandle<>0) then SendMessage(FHighEditHandle, wm_EndSession, Msg.wParam, Msg.lParam);
end;

{----------------------------------------------------------------------}

procedure THighEdit.WMEnterIdle(var Msg: TMessage);
begin
  IBMHideDropDown;
end;

{----------------------------------------------------------------------}

procedure THighEdit.WMMove(var Msg: TMessage);
begin
  IBMHideDropDown;
end;

{----------------------------------------------------------------------}

procedure THighEdit.WMSetFocus(var Msg: TMessage);
begin
  if (FHighEditHandle<>0) then WinProcs.SetFocus(FHighEditHandle);
end;

{----------------------------------------------------------------------}

procedure THighEdit.WMSize(var Msg: TMessage);
begin
  Inherited;
  Resize;
end;

{----------------------------------------------------------------------}

procedure THighEdit.WMWinIniChange(var Msg: TMessage);
var
  clf: TColorLogFont;
begin
  if (FHighEditHandle<>0) then SendMessage(FHighEditHandle, wm_WinIniChange, Msg.wParam, Msg.lParam);
  if (FIconHandle<>0) then begin
    SendMessage(FIconHandle, wm_WinIniChange, Msg.wParam, Msg.lParam);
    if (FHighEditHandle<>0) then begin
      if HEIsScreenFont then begin
        HEGetScreenColorLogFont(Word(-1), @clf);
        IBMSetColorLogFont(font_Display, @clf);
      end
      else begin
        HEGetColorLogFont(Word(-1), @clf);
        IBMSetColorLogFont(font_Printer, @clf);
      end;
    end;
  end;
end;


(*** HE ********************************************************)


function  THighEdit.HECursorHome: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_CursorHome, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEAddAllPFFlags(AFlag: Byte; ATypes: PWord): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_AddAllPFFlagsExt, MakeWord(byte(-1), AFlag), longint(ATypes)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEAddFunctionPF(ATyp: Word; AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_AddFunctionPF, ATyp, longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEAddPFFlags(AFlag: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_AddPFFlags, AFlag, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEAdjustTable(AFlag: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_AdjustTable, AFlag, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEAppend: Boolean;
var
  Name: array [0..255] of Char;
begin
  StrCopy(Name, FFileBoxParamPtr^.szPathName);
  StrCat(Name, FFileBoxParamPtr^.szFileName);
  Result := HEAppendExt(FFileBoxParamPtr^.wFileType, Name);
end;

{----------------------------------------------------------------------}

function THighEdit.HEAppendAnsi(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_AppendAnsi, 0, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEAppendData(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_AppendData, 0, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEAppendExt(AFileType: Word; AName: PChar): Boolean;
begin
  case AFileType of
    FileFormat_ANSI: Result := HEAppendAnsi(AName);
    FileFormat_OEM:  Result := HEAppendOEM(AName);
    FileFormat_RTF:  Result := HEAppendRTF(AName);
    else             Result := HEAppendData(AName);
  end;
end;

{----------------------------------------------------------------------}

function THighEdit.HEAppendOEM(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_AppendOEM, 0, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEAppendRTF(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_AppendRTF, 0, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HECanCopy: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_CanCopy, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HECanPaste: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_CanPaste, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HECanUndo: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_CanUndo, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEChangeFunctionPF(AType: Word; ABuff: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_ChangeFunctionPF, AType, longint(ABuff)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEChangeStyle(AStyle: Word; ASwitch: Boolean): Boolean;
var
  Style: Word;
begin
  Style := HEGetStyle;
  if (ASwitch)
  then Style := Style or AStyle
  else Style := Style and not AStyle;
  Result := HESetStyle(Style, current_Color)
end;

{----------------------------------------------------------------------}

function THighEdit.HEClearAllPFFlags(AFlag: Byte; ATypes: PWord): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_ClearAllPFFlagsExt,
            MakeWord(byte(-1), AFlag), longint(ATypes)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEClearPFFlags(AFlag: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_ClearPFFlags, AFlag, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEClearUndoList: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_ClearUndoList, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HECloseActivePF(ARepaint: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_CloseActivePF, Word(ARepaint), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HECloseActiveTable: Boolean;
begin
  Result:= Boolean(SendMessage(FHighEditHandle, he_CloseActiveTable, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HECopyData: LongInt;
begin
  Result := THandle(SendMessage(FHighEditHandle, heCopyData, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HECopyRTF: THandle;
begin
  Result := THandle(SendMessage(FHighEditHandle, he_CopyRTF, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HECountToken(AMode: Word; AText: PChar): Longint;
begin
  Result := SendMessage(FHighEditHandle, he_CountToken, AMode, Longint(AText));
end;

{----------------------------------------------------------------------}

function THighEdit.HECursorEnd: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_CursorEnd, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEDeletePF(ARepaint: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_DeletePF, MakeWord(0,Byte(ARepaint)), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEDeleteTab(AOffset: longint): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_DeleteTab, 0, AOffset));
end;

{----------------------------------------------------------------------}

function THighEdit.HEDeleteTableRow(ARow: Longint): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_DeleteTableRow, 0, ARow));
end;

{----------------------------------------------------------------------}

function THighEdit.HEEnableScreenFonts(ASwitch: Boolean): Boolean;
begin
  Result:= Boolean(SendMessage(FHighEditHandle, he_EnableScreenFonts, word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEError (wID: Word; AType: TMsgDlgType; AButtons: TMsgDlgButtons) : Word;
begin
  Result := MessageDlg(LoadStr(wID), AType, AButtons, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEExtSetRedraw(AScreen, ARepaint, ASave: Boolean): Boolean;
begin
  Result := Boolean(SendMessage( FHighEditHandle, he_ExtSetRedraw, Word(AScreen),
            MakeLong(Word(ARepaint),Word(ASave))));
end;

{----------------------------------------------------------------------}

function THighEdit.HEFmtLines(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_FmtLines, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEFormat: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_Format, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEFormFeed: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_FormFeed, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetAlign: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetAlign, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetChar: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetChar, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetCharLeft: Char;
begin
  Result := char(LoByte(HEGetChar));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetCharRight: Char;
begin
  Result := char(HiByte(HEGetChar));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetColor: TColorRef;
begin
  Result := TColorRef(SendMessage(FHighEditHandle, he_GetColor, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetColorLogFont(AIndex: Word; APointer: PColorLogFont): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_GetColorLogFont, AIndex, longint(APointer)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetContext: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetContext, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetCursor: PLongint;
begin
  Result := PLongint(SendMessage(FHighEditHandle, he_GetCursor, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetCursorX: Longint;
begin
  Result := GetXPos(HEGetCursor);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetCursorY: Longint;
begin
  Result := GetYPos(HEGetCursor);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetDocInfo(APointer: PGetDocInfo): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_GetDocInfo, 0, longint(APointer)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetEofChar: Char;
begin
  Result := Char(SendMessage(FHighEditHandle, he_GetEofChar, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetFontHandle(AIndex: Word; AFlag: longint): HFont;
begin
  Result := HFont(SendMessage(FHighEditHandle, he_GetFontHandle, AIndex, AFlag));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetFontIndex: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetFontIndex, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetFootHWnd: HWnd;
begin
  Result := HiWord(HEGetHeadFootHWnd);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetFootRect: PRect;
begin
  Result := PRect(SendMessage(FHighEditHandle, he_GetFootRect, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetFragment: PTextFragment;
begin
  Result := PTextFragment(SendMessage(FHighEditHandle, he_GetFragment, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetFunctionPFTyp(AIndex: word): Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetFunctionPFTyp, AIndex, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetFunctionTableCount: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetFunctionTableCount, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetHeadFootHWnd: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetHeadFootHWnd, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetHeadHWnd: HWnd;
begin
  Result := LoWord(HEGetHeadFootHWnd);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetHeadRect: PRect;
begin
  Result := PRect(SendMessage(FHighEditHandle, he_GetHeadRect, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetInstancePrinter: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetInstancePrinter, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetInstancePrinterDevMode: THandle;
begin
  Result := HiWord(HEGetInstancePrinter);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetInstancePrinterDevNames: THandle;
begin
  Result := LoWord(HEGetInstancePrinter);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetLine(AMax: Word; ABuff: PChar): Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetLine, AMax, longint(ABuff)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetLineCount: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetLineCount, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetMargins: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetMargins, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetMarkedText(ABuff: PChar; AMaxSize: Word): PChar;
var
  Handle  : TGlobalHandle;
  TmpBuff : PChar;
  TmpSize : Word;
begin
  StrCopy(ABuff, '');
  Result := ABuff;
  Handle := HEGetTextSel;
  if (Handle<>0) then begin
    TmpBuff := GlobalLock(Handle);
    if (TmpBuff<>nil) then StrLCopy(ABuff, TmpBuff, AMaxSize);
    GlobalFree(Handle);
    GlobalUnlock(Handle);
  end;
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetNotifyWParam: Word;
begin
  Result := SendMessage(FHighEditHandle, he_GetNotifyWParam, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetNotifyLParam: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetNotifyLParam, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPage: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPage, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPageCursor: Word;
begin
  Result := LoWord(HEGetPage);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPageMax: Word;
begin
  Result := HiWord(HEGetPage);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPFBkColor: TColorRef;
begin
  Result := TColorRef(SendMessage(FHighEditHandle, he_GetPFBkColor, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPFData(ABuff: PChar): Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetPFData, 0, longint(ABuff)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPFDataLen: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetPFDataLen, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPFFlags: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetPFFlags, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPFWidth: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPFWidth, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPhysicalDim: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPhysicalDim, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPhysicalMargins: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPhysicalMargins, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPictureDim: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPictureDim, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPictureFlags: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetPictureFlags, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPictureOffset: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPictureOffset, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPrinterCursorHeight: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPrinterCursorHeight, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPrinterDC: HDC;
begin
  Result := HDC(SendMessage(FHighEditHandle, he_GetPrinterDC, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPrinterDim: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPrinterDim, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPrinterDimX: Word;
begin
  Result := LoWord(HEGetPrinterDim);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPrinterDimY: Word;
begin
  Result := HiWord(HEGetPrinterDim);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPrinterIC: HDC;
begin
  Result := HDC(SendMessage(FHighEditHandle, he_GetPrinterIC, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPrinterLineHeight: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPrinterLineHeight, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPrinterLineWidth: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPrinterLineWidth, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetPrinterTextHeight: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetPrinterTextHeight, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetRBMargins: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetRBMargins, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetRGB(AIndex: Word): Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetRGB, AIndex, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetScreenColorLogFont(AIndex: Word; APointer: PColorLogFont): Boolean;
begin
  Result := Boolean( SendMessage(FHighEditHandle,
            he_GetScreenColorLogFont, AIndex, longint(APointer)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetScrollPos: longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetScrollPos, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetSelection: PLongint;
begin
  Result := PLongint(SendMessage(FHighEditHandle, he_GetSelection, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetSelectionStart: PLongint;
begin
  Result := PLongint(SendMessage(FHighEditHandle, he_GetSelectionStart, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetSelectionStartX: Longint;
begin
  Result := GetXPos(HEGetSelectionStart);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetSelectionStartY: Longint;
begin
  Result := GetYPos(HEGetSelectionStart);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetSelectionX: Longint;
begin
  Result := GetXPos(HEGetSelection);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetSelectionY: Longint;
begin
  Result := GetYPos(HEGetSelection);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetSpacing: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetSpacing, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetString(AMax: Word; ABuff: PChar): Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetString, AMax, longint(ABuff)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetStyle: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetStyle, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetTab(ATabHandlePtr: PHandle): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_GetTab, 0, Longint(ATabHandlePtr)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetTabWidth: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetTabWidth, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetTextDim: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetTextDim, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetTextSel: THandle;
var
  Handle  : TGlobalHandle;
begin
  if Boolean(SendMessage(FHighEditHandle, he_GetTextSel, 0, Longint(@Handle)))
  then Result := Handle
  else Result := 0;
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetTextWidth: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetTextWidth, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetTyp: Byte;
begin
  Result := Byte(SendMessage(FHighEditHandle, he_GetTyp, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetUndoDepth: Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_GetUndoDepth, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEGetUnit: Longint;
begin
  Result := SendMessage(FHighEditHandle, he_GetUnit, 0, 0);
end;

{----------------------------------------------------------------------}

function THighEdit.HEInsertBitmap(ABitmap: HBitmap): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_InsertBitmap, ABitmap, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEInsertFunctionPFParam(ATyp: word; AFlag: word; ABuff: PChar): Boolean;
var
  IFPF: TInsertFunctionPFParam;
begin
  IFPF.wTyp   := ATyp;
  IFPF.wFlags := AFlag;
  IFPF.pData  := ABuff;
  Result := Boolean(SendMessage(FHighEditHandle, he_InsertFunctionPFParam, 0, longint(@IFPF)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEInsertTab(ATyp: Byte; APos: Word): Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_InsertTab, ATyp, APos));
end;

{----------------------------------------------------------------------}

function THighEdit.HEInsertTable(ACol, ARow: Word): HWnd;
begin
  Result := HWnd(SendMessage(FHighEditHandle, he_InsertTable, 0, MakeLong(ACol, ARow)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEInsertTableRow(ARow: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_InsertTableRow, 0, ARow));
end;

{----------------------------------------------------------------------}

function THighEdit.HEInvertStyle(AStyle: Word): Boolean;
var
  Style: Word;
begin
  Style := HEGetStyle;
  if ((Style and AStyle)<>0)
  then Style := Style and not AStyle
  else Style := Style or AStyle;
  Result := HESetStyle(Style, current_Color)
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsClearDelete: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsClearDelete, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsCrVisible: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsCrVisible, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsFormfeed: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsFormfeed, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsGrayBk: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsGrayBk, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsHeadFootVisible: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsHeadFootVisible, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsIgnoreEditable: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsIgnoreEditable, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsInsertMode: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsInsertMode, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsLineBreak: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsLineBreak, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsLineEditable: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsLineEditable, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsLineGhost: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsLineGhost, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsModified: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsModified, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsPercentThumb: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsPercentThumb, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsPFActive: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsPFActive, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsPFBkVisible: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsPFBkVisible, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsPFFunctionVisible: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsPFFunctionVisible, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsPFGhost: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsPFGhost, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsPictureFrame: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsPictureFrame, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsPreview: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsPreview, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsReadOnly: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsReadOnly, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsScreenFont: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsScreenFont, Word(-1), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsScreenFontsEnabled: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsScreenFontsEnabled, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsSpaceVisible: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsSpaceVisible, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsTableActive: HWnd;
begin
  Result := HWnd(SendMessage(FHighEditHandle, he_IsTableActive, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsTabVisible: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsTabVisible, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEIsTextSelectable: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_IsTextSelectable, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HELineLength(ARow: Longint): Longint;
begin
  Result := SendMessage(FHighEditHandle, he_LineLength, 0, ARow);
end;

{----------------------------------------------------------------------}

function THighEdit.HELoad: Boolean;
var
  Name: array [0..255] of Char;
begin
  StrCopy(Name, FFileBoxParamPtr^.szPathName);
  StrCat(Name, FFileBoxParamPtr^.szFileName);
  Result := HELoadExt(FFileBoxParamPtr^.wFileType, Name);
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadAnsi(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_LoadAnsi, 0, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadBMP(AFlag: Word; AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_LoadBitmap, AFlag, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadData(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_LoadData, 0, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadExt(AFileType: Word; AName: PChar): Boolean;
begin
  case AFileType of
    FileFormat_ANSI: Result := HELoadAnsi(AName);
    FileFormat_OEM:  Result := HELoadOEM(AName);
    FileFormat_RTF:  Result := HELoadRTF(AName);
    else             Result := HELoadData(AName);
  end;
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadGIF(AFlag: Word; AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_LoadGIF, AFlag, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadOEM(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_LoadOEM, 0, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadPCX(AFlag: Word; AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_LoadPCX, AFlag, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadPicture: Boolean;
var
  Name: array [0..255] of Char;
begin
  StrCopy(Name, FPictureFileBoxParamPtr^.szPathName);
  StrCat(Name, FPictureFileBoxParamPtr^.szFileName);
  Result := HELoadPictureExt(FFileBoxParamPtr^.wFileType, Name);
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadPictureExt(AFileType: Word; AName: PChar): Boolean;
begin
  case AFileType of
    FileFormat_PCX:    Result := HELoadPCX(bmp_Scale, AName);
    FileFormat_GIF:    Result := HELoadGIF(bmp_Scale, AName);
    FileFormat_TIFF:   Result := HELoadTIF(bmp_Scale, AName);
    FileFormat_TARGA:  Result := HELoadTGA(bmp_Scale, AName);
    FileFormat_WMF:    Result := HELoadWMF(bmp_Scale, AName);
    else               Result := HELoadBMP(bmp_Scale, AName);
  end;
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadRTF(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_LoadRTF, 0, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadTGA(AFlag: Word; AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_LoadTarga, AFlag, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadTIF(AFlag: Word; AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_LoadTiff, AFlag, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HELoadWMF(AFlag: Word; AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_LoadWMF, AFlag, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HELockFunctionPF(ATyp: Word): PChar;
begin
  Result := PChar(SendMessage(FHighEditHandle, he_LockFunctionPF, ATyp, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEMoveMargin(AParam: PMoveMarginParam): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_MoveMargin, 0, Longint(AParam)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEMoveTab(APointer: PMoveTabParam): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_MoveTab, 0, Longint(APointer)));
end;

{----------------------------------------------------------------------}

function THighEdit.HENew: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_New, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HENextPF(ASearch: Byte; ATypes: PWord): Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_NextPFExt, MakeWord(0, ASearch), longint(ATypes)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEPaginate(AHeadHandle, AFootHandle: HWnd): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_Paginate, 0, MakeLong(AHeadHandle, AFootHandle)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEPasteData(AHandle: THandle): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_PasteData, 0, AHandle));
end;

{----------------------------------------------------------------------}

function THighEdit.HEPasteRTF(AHandle: THandle): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_PasteRTF, 0, AHandle));
end;

{----------------------------------------------------------------------}

function THighEdit.HEPreview: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_Preview, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEPrintDocument(AMode: Word; APtr: PPrintDocParam): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_PrintDocument, AMode, longint(APtr)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEPrintFragment(APtr: PTextFragment): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_PrintFragment, 0, longint(APtr)));
end;

{----------------------------------------------------------------------}

function THighEdit.HEQueryUndoType(AIndex: Word): Longint;
begin
  Result := Word(SendMessage(FHighEditHandle, he_QueryUndoType, AIndex, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEQuickreplace(ALng: Word; AData: pointer): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_Quickreplace, ALng, Longint(AData)));
end;

{----------------------------------------------------------------------}

function THighEdit.HERedraw: Boolean;
var
  hwndUpdate: HWND;
begin
  case HEGetContext of
    context_Main: hwndUpdate := FHighEditHandle;
    context_Head: hwndUpdate := HEGetHeadHWnd;
    context_Foot: hwndUpdate := HEGetFootHWnd;
  end;
  InvalidateRect(hwndUpdate, nil, false);
  UpdateWindow(hwndUpdate);
  Result := true;
end;

{----------------------------------------------------------------------}

function THighEdit.HEReplaceSel(ABuff: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_replacesel, 0, Longint(ABuff)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESave: Boolean;
var
  Name: array [0..255] of Char;
begin
  StrCopy(Name, FFileBoxParamPtr^.szPathName);
  StrCat(Name, FFileBoxParamPtr^.szFileName);
  Result := HESaveExt(FFileBoxParamPtr^.wFileType, Name);
end;

{----------------------------------------------------------------------}

function THighEdit.HESaveAnsi(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SaveAnsi, of_DefHESave, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESaveData(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SaveData, of_DefHESave, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESaveExt(AFileType: Word; AName: PChar): Boolean;
begin
  case AFileType of
    FileFormat_ANSI: Result := HESaveAnsi(AName);
    FileFormat_OEM:  Result := HESaveOEM(AName);
    FileFormat_RTF:  Result := HESaveRTF(AName);
    else             Result := HESaveData(AName);
  end;
end;

{----------------------------------------------------------------------}

function THighEdit.HESaveOEM(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SaveOEM, of_DefHESave, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESaveRTF(AName: PChar): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SaveRTF, of_DefHESave, Longint(AName)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESearch(AFlags: Word; AText: PChar): Word;
begin
  Result := Word(SendMessage(FHighEditHandle, he_Search, AFlags, Longint(AText)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESelectAll: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SelectAll, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESelectWord: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SelectWord, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetAbortDlg: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetAbortDlg, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetAlign(AAlign: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetAlign, AAlign, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetAllPFFlags(AFlag: Byte; ATypes: PWord): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetAllPFFlagsExt, MakeWord(0, AFlag), longint(ATypes)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetAllPFWidth(AWidth: Word; ATypes: PWord): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetAllPFWidthExt, AWidth, longint(ATypes)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetAutoHSize(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetAutoHSize, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetClearDelete(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetClearDelete, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetColorLogFont(AFlag: Byte; APointer: PColorLogFont): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetColorLogFont, MakeWord(AFlag, 0), longint(APointer)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetContext(AContext: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetContext, AContext, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetCrVisible(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetCrVisible, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetCursor(ACol: Word; ARow:Longint): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetCursor, ACol, ARow));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetDocInfo(APointer: PSetDocInfo): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetDocInfo, 0, longint(APointer)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetEOFChar(AChar: char): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetEOFChar, Word(AChar), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetGrayBk(AGray: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetGrayBk, Word(AGray), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetHScroll(AWindow: HWND): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetHScroll, AWindow, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetIgnoreEditable(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetIgnoreEditable, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetInsert(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetInsert, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetInstancePrinter(ADevNames, ADevMode: THandle): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle,
            he_SetInstancePrinter, 0, MakeLong(ADevNames, ADevMode)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetLeftMargin(AWidth: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetLeftMargin, AWidth, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetLineBreak(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetLineBreak, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetLineeditable(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetLineeditable, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetLineGhost(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetLineGhost, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetMargins(AX, AY: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetMargins, 0, MakeLong(AX,AY)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetModified(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetModified, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetMouseCursor(ACursor: HCursor): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetMouseCursor, ACursor, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetOffset(ARepaint: Boolean; AX, AY: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetOffset, Word(ARepaint), MakeLong(AX, AY)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPageCounter(ACounter: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPageCounter, ACounter, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPercentThumb(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPercentThumb, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPFBkColor(AColor: TColorRef): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPFBkColor, 0, AColor));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPFBkVisible(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPFBkVisible, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPFFlags(AFlag: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPFFlags, AFlag, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPFFunctionVisible(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPFFunctionVisible, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPFGhost(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPFGhost, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPFWidth(AWidth: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPFWidth, AWidth, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPictureDim(AWidth, AHeight: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPictureDim, 0, MakeLong(AWidth, AHeight)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPictureFlags(AFlags: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPictureFlags, AFlags, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPictureFrame(AAsFrame: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPictureFrame, Word(AAsFrame), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetPictureOffset(AX, AY: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetPictureOffset, 0, MakeLong(AX,AY)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetRBMargins(AX, AY: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetRBMargins, 0, MakeLong(AX,AY)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetReadOnly(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetReadOnly, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetScrollPos(APos: longint): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetScrollPos, 0, APos));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetSelection(ACol: Word; ARow:Longint): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetSelection, ACol, ARow));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetSelectionEditable(AEdit: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetSelectionEditable, Word(AEdit), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetSpaceVisible(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetSpaceVisible, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetSpacing(ASpacing: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetSpacing, ASpacing, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetStyle(AStyle: Word; AColor: TColorRef): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetStyle, AStyle, AColor));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetTab(ATabPtr: PTab): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetTab, of_DefHESave, Longint(ATabPtr)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetTabVisible(ASwitch: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetTabVisible, Word(ASwitch), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetTabWidth(AWidth: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetTabWidth, AWidth, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetTextDim(AWidth, AHeight: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetTextDim, AWidth, MakeLong(AWidth,AHeight)));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetTextHeight(AHeight: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetTextHeight, AHeight, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetTextSelectable(Value: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetTextSelectable, Word(Value), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetTextWidth(AWidth: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetTextWidth, AWidth, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetTopMargin(AHeight: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetTopMargin, AHeight, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetUndoDepth(ADepth: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetUndoDepth, ADepth, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetUnit(AUnit: Word): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetUnit, AUnit, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HESetVScroll(AWindow: HWND): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_SetVScroll, AWindow, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEShowHeadFoot(ABool: Boolean): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_ShowHeadFoot, Word(ABool), 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEStatusLine(AWindow: HWnd; AStatusLinePtr: PStatusLine): Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_StatusLine, AWindow, Longint(AStatusLinePtr)));
end;

{----------------------------------------------------------------------}

function THighEdit.HETableActivate: HWnd;
begin
  Result := HWnd(SendMessage(FHighEditHandle, he_TableActivate, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEUndo: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_Undo, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEUnlockFunctionPF: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_UnlockFunctionPF, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.HEWriteProfile: Boolean;
begin
  Result := Boolean(SendMessage(FHighEditHandle, he_WriteProfile, 0, 0));
end;

{----------------------------------------------------------------------}

function THighEdit.IBMEnableScreenFonts(ASwitch: Boolean): Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_EnableScreenFonts, Word(ASwitch), 0))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMGetAlign: Word;
begin
  if (FIconHandle<>0)
  then Result := Word(SendMessage(FIconHandle, ibm_GetAlign, 0, 0))
  else Result := 0;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMGetColor: TColorRef;
begin
  if (FIconHandle<>0)
  then Result := TColorRef(SendMessage(FIconHandle, ibm_GetColor, 0, 0))
  else Result := 0;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMGetColorLogFont(APointer: PColorLogFont): Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_GetColorLogFont, 0, longint(APointer)))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMGetDisplay: Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_GetDisplay, 0, 0))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMGetSpacing: Word;
begin
  if (FIconHandle<>0)
  then Result := Word(SendMessage(FIconHandle, ibm_GetSpacing, 0, 0))
  else Result := 0;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMGetStyle: Word;
begin
  if (FIconHandle<>0)
  then Result := Word(SendMessage(FIconHandle, ibm_GetStyle, 0, 0))
  else Result := 0;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMGetTab: Word;
begin
  if (FIconHandle<>0)
  then Result := Word(SendMessage(FIconHandle, ibm_GetTab, 0, 0))
  else Result := 0;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMHideDropDown: Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_HideDropDown, 0, 0))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMIsScreenFont: Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_IsScreenFont, 0, 0))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMSetAlign(AAlign: Word): Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_SetAlign, AAlign, 0))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMSetColorLogFont(AFlag: Byte; APointer: PColorLogFont): Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage( FIconHandle, ibm_SetColorLogFont, AFlag, longint(APointer)))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMSetDisplay(ASwitch: Boolean): Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_SetDisplay, Word(ASwitch), 0))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMSetFontSize(ASize: Word): Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_SetFontSize, ASize, 0))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMSetSpacing(ASpacing: Word): Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_SetSpacing, ASpacing, 0))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMSetStyle(AStyle: Word): Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_SetStyle, AStyle, 0))
  else Result := false;
end;

{----------------------------------------------------------------------}

function THighEdit.IBMSetTab(ATab: Word): Boolean;
begin
  if (FIconHandle<>0)
  then Result := Boolean(SendMessage(FIconHandle, ibm_SetTab, ATab, 0))
  else Result := false;
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMAlign(var Msg: TMessage);
begin
  IBMSetAlign(HEGetAlign);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMChar(var Msg: TMessage);
var
  Message: TMessage;
begin
  Message.Msg := WM_CHAR;
  Message.WParam := HEGetNotifyWParam;
  Message.LParam := HEGetNotifyLParam;
  Dispatch(Message);
  if Char(Message.WParam) = #0 then Msg.Result := 1;
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMClick(var Msg: TMessage);
begin
  if Assigned(FOnClick) then FOnClick(Self);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMCursorPos(var Msg: TMessage);
var
  szBuffer: array[0..127] of Char;
  Page    : array[0..20] of Char;
begin
  if (FStatusHandle<>0) then begin
    Str(HEGetPageCursor+1, szBuffer);
    Str(HEGetPageMax,      Page);
    StrCat(szBuffer, '/');
    StrCat(szBuffer, Page);
    SendMessage(FStatusHandle, sl_SetItemText, 4, Longint (@szBuffer));
    Str(HEGetCursorY, szBuffer);
    SendMessage(FStatusHandle, sl_SetItemText, 2, Longint (@szBuffer));
    Str(HEGetCursorX, szBuffer);
    SendMessage(FStatusHandle, SL_SetItemText, 0, Longint (@szBuffer));
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMDblClk(var Msg: TMessage);
begin
  if FDblClickSelectWord then HESelectWord;
  if Assigned(FOnDblClick) then FOnDblClick(Self);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMEndPF(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMFileExists(var Msg: TMessage);
begin
  if FDoSaveAs then begin
    if HEError(ids_ReplaceFile,  mtConfirmation, [mbYes, mbNo]) = id_No
    then Msg.Result := 1
    else Msg.Result := 0;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMFont(var Msg: TMessage);
var
  clf: TColorLogFont;
begin
  if (FIconHandle<>0) then begin
    if HEIsScreenFont then begin
      SendMessage(LoWord (Msg.lParam), he_GetScreenColorLogFont, Word (-1), Longint (@clf));
      IBMSetColorLogFont(font_Display, @clf);
    end
    else begin
      SendMessage(LoWord(Msg.lParam), he_GetColorLogFont, Word(-1), LongInt(@clf));
      IBMSetColorLogFont(font_Printer, @clf);
    end;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMFontSize(var Msg: TMessage);
var
  clf: TColorLogFont;
begin
  if (FIconHandle<>0) then begin
    if HEIsScreenFont
    then HEGetScreenColorLogFont(Word(-1), @clf)
    else HEGetColorLogFont(Word(-1), @clf);
    IBMSetFontSize(clf.wHeight);
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMFormatPF(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMHScroll(var Msg: TMessage);
var
  Width    : Integer;
  ScrollPos: Longint;
begin
  Width := Integer (SendMessage(LoWord (Msg.lParam), he_GetNotifyScrollWidth, 0, 0));
  ScrollPos := SendMessage(LoWord (Msg.lParam), he_GetNotifyHScrollPos, 0, 0);
  if (FTabHandle<>0) then SendMessage(FTabHandle, tm_HScroll, Width, ScrollPos);
  if (FRulerHandle<>0) then SendMessage(FRulerHandle, rl_HScroll, Width, 0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMInitNew(var Msg: TMessage);
begin
  UnitType := utCentimeter;
  if Assigned(FOnInitNew) then OnInitNew(self);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMInsert(var Msg: TMessage);
begin
  if SendMessage(LoWord (Msg.lParam), he_GetInsert, 0, 0) <> 0
  then HESetInsert(false)
  else HESetInsert(true);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMInsertPF(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMKeyDown(var Msg: TMessage);
var
  Message: TMessage;
begin
  Message.Msg := WM_KEYDOWN;
  Message.WParam := HEGetNotifyWParam;
  Message.LParam := HEGetNotifyLParam;
  Dispatch(Message);
  if Message.WParam = 0 then Msg.Result := 1;
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMKeyUp(var Msg: TMessage);
begin
  Perform(WM_KEYUP, HEGetNotifyWParam, HEGetNotifyLParam);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMKillFocus(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMLButtonDown(var Msg: TMessage);
begin
  Perform(WM_LBUTTONDOWN, HEGetNotifyWParam, HEGetNotifyLParam);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMLButtonUp(var Msg: TMessage);
begin
  Perform(WM_LBUTTONUP, HEGetNotifyWParam, HEGetNotifyLParam);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMMButtonDown(var Msg: TMessage);
begin
  Perform(WM_MBUTTONDOWN, HEGetNotifyWParam, HEGetNotifyLParam);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMMButtonUp(var Msg: TMessage);
begin
  Perform(WM_MBUTTONUP, HEGetNotifyWParam, HEGetNotifyLParam);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMMessageText(var Msg: TMessage);
var
  Buff   : PChar;
  BuffLng: Word;
begin
  if (FStatusHandle<>0) then begin
    Buff    := PChar (SendMessage(LoWord (Msg.lParam), he_GetNotifyMessageText, 0, 0));
    BuffLng := Word (SendMessage(LoWord (Msg.lParam), he_GetNotifyMessageLen, 0, 0));
    SendMessage(FStatusHandle, sl_DisplayText, BuffLng, Longint (Buff));
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMMouseMove(var Msg: TMessage);
begin
  Perform(WM_MOUSEMOVE, HEGetNotifyWParam, HEGetNotifyLParam);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMNewPageFragment(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMPrintFoot(var Msg: TMessage);
begin
  Msg.Result := Longint(not FPrintFoot);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMPrintHead(var Msg: TMessage);
begin
  Msg.Result := Longint(not FPrintHead);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMRButtonDown(var Msg: TMessage);
begin
  Perform(WM_RBUTTONDOWN, HEGetNotifyWParam, HEGetNotifyLParam);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMRButtonUp(var Msg: TMessage);
begin
  Perform(WM_RBUTTONUP, HEGetNotifyWParam, HEGetNotifyLParam);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMSelectPF(var Msg: TMessage);
var
  Flag                : Word;
  result              : Integer;
begin
{    Flag := HEGetPFFlags;
    result := Application.ExecDialog(new(PFieldFlagsDialog, Init(@self, @Flag)));
    if (result=id_OK) then begin
        HESetPFFlags(Flag);
        HERedraw;
    end;}
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMSelectPicture(var Msg: TMessage);
{var
  Flag                : Word;
  result              : Integer;}
begin
{  Flag  := HEGetPictureFlags;
  result := Application^.ExecDialog(new(PPictureFlagsDialog, Init(@self, @Flag)));
  if (result=id_OK) then HESetPictureFlags(Flag);}
  if Assigned(FOnPictureSelect) then OnPictureSelect(self);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMSetFocus(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMSetHScrollPos(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMSetHScrollRange(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMSetVScrollPos(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMSetVScrollRange(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMSizePF(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMSpacing(var Msg: TMessage);
begin
  IBMSetSpacing(HEGetSpacing);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMStyle(var Msg: TMessage);
begin
  IBMSetStyle(HEGetStyle);
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMTabModify(var Msg: TMessage);
var
  pTabList: PTab;
begin
  if (FTabHandle<>0) then begin
    pTabList := PTab (SendMessage(LoWord (Msg.lParam), he_GetNotifyTabList, 0, 0));
    SendMessage(FTabHandle, tm_SetTabList, 1, Longint (pTabList));
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMTabWidth(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.HEMVScroll(var Msg: TMessage);
begin
end;

{----------------------------------------------------------------------}

procedure THighEdit.IBNAlign(var Msg: TMessage);
begin
  HESetAlign(IBMGetAlign);
end;

{----------------------------------------------------------------------}

procedure THighEdit.IBNColor(var Msg: TMessage);
begin
  FColorBoxParamPtr^.cr := IBMGetColor;
  HESetStyle(current_Attribute, FColorBoxParamPtr^.cr);
end;

{----------------------------------------------------------------------}

procedure THighEdit.IBNDblClk(var Msg: TMessage);
begin
  HEGetColorLogFont(Word(-1), @FFontBoxParamPtr^.clf);
  if FontDialogBox (Handle, FFontBoxParamPtr)
  then HESetColorLogFont(font_Printer + font_Set_All, @FFontBoxParamPtr^.clf);
end;

{----------------------------------------------------------------------}

procedure THighEdit.IBNDisplay(var Msg: TMessage);
begin
  Redraw := False;
  HESetCRVisible(IBMGetDisplay);
  HESetSpaceVisible(IBMGetDisplay);
  HESetTabVisible(IBMGetDisplay);
  Redraw := True;
end;

{----------------------------------------------------------------------}

procedure THighEdit.IBNFontName(var Msg: TMessage);
var
  clf: TColorLogFont;
begin
  IBMGetColorLogFont(@clf);
  if IBMIsScreenFont
  then HESetColorLogFont(font_Display + font_Set_Name, @clf)
  else HESetColorLogFont(font_Printer + font_Set_Name, @clf);
end;

{----------------------------------------------------------------------}

procedure THighEdit.IBNFontSize(var Msg: TMessage);
var
  clf: TColorLogFont;
begin
  IBMGetColorLogFont(@clf);
  if IBMIsScreenFont
  then HESetColorLogFont(font_Display + font_Set_Size, @clf)
  else HESetColorLogFont(font_Printer + font_Set_Size, @clf);
end;

{----------------------------------------------------------------------}

procedure THighEdit.IBNSpacing(var Msg: TMessage);
begin
  HESetSpacing(IBMGetSpacing);
end;

{----------------------------------------------------------------------}

procedure THighEdit.IBNStyle(var Msg: TMessage);
begin
  HESetStyle(IBMGetStyle, current_Color);
end;

{----------------------------------------------------------------------}

procedure THighEdit.IBNTab(var Msg: TMessage);
var
  TabTyp: Byte;
begin
  TabTyp := IBMGetTab;
  SendMessage(FTabHandle, tm_SetTyp, TabTyp, 0);
end;

{----------------------------------------------------------------------}

procedure THighEdit.IDStatus(var Msg: TMessage);
var
  StatusHeight        : Integer;
  rc                  : TRect;
begin
  if HiWord (Msg.lParam) = sln_NewHeight then begin
     StatusHeight := SendMessage(LoWord (Msg.lParam), sl_GetHeight, 0, 0);
     rc := GetClientRect;
     MoveWindow ( LoWord (Msg.lParam), 0, rc.bottom - StatusHeight,
                  rc.right, StatusHeight, true);
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.RNLButtonDblClk(var Msg: TMessage);
begin
  ParagraphDialogBox (Handle, FParagraphBoxParamPtr);
end;

{----------------------------------------------------------------------}

procedure THighEdit.RNRButtonDblClk(var Msg: TMessage);
begin
  ParagraphDialogBox (Handle, FParagraphBoxParamPtr);
end;

{----------------------------------------------------------------------}

procedure THighEdit.TNIntDelete(var Msg: TMessage);
var
  Return: Longint;
begin
  Return := SendMessage(LoWord (Msg.lParam), tm_GetPos, 0, 0);
  HEDeleteTab(Tab_GetPrinterPos(PLongint(Return)));
end;

{----------------------------------------------------------------------}

procedure THighEdit.TNIntInsert(var Msg: TMessage);
var
  Typ                 : Byte;
  Return              : Longint;
  UnitOffset          : Longint;
  NewOffset           : array [0..1] of Longint;
begin
  Typ          := Byte (SendMessage(LoWord (Msg.lParam), tm_GetTyp, 0, 0));
  Return       := SendMessage(LoWord (Msg.lParam), tm_GetPos, 0, 0);
  UnitOffset   := Tab_GetUnitPos(PLongint(Return));
  NewOffset[0] := HEInsertTab(Typ, UnitOffset);
  NewOffset[1] := 0;
  SendMessage(LoWord (Msg.lParam), tm_SetPrinterOffset, 0, Longint (@NewOffset[0]));
end;

{----------------------------------------------------------------------}

procedure THighEdit.TNIntUpdate(var Msg: TMessage);
var
  Return              : Longint;
  NewOffset           : array [0..1] of Longint;
  mmp                 : TMoveMarginParam;
  mtp                 : TMoveTabParam;
begin
  Return := SendMessage(LoWord (Msg.lParam), tm_GetPos, 0, 0);
  if ((HiWord (Msg.lParam) = tn_UpdateLM)
    or (HiWord (Msg.lParam) = tn_UpdateRM)
    or (HiWord (Msg.lParam) = tn_UpdateIndent)) then begin
    mmp.lUnitOffset := Tab_GetUnitPos (PLongint (Return));
    mmp.lPrinterOffset := Tab_GetPrinterPos (PLongint (Return));
    mmp.lIndentOffset := 0;
    mmp.byTyp := LoByte (HiWord (Msg.lParam));
    HEMoveMargin(@mmp);
    NewOffset[0] := mmp.lPrinterOffset;
    NewOffset[1] := mmp.lIndentOffset;
  end
  else begin
    mtp.lUnitOffset    := Tab_GetUnitPos(PLongint(Return));
    mtp.lPrinterOffset := Tab_GetPrinterPos(PLongint(Return));
    HEMoveTab(@mtp);
    NewOffset[0] := mtp.lUnitOffset;
    NewOffset[1] := mtp.lPrinterOffset;
  end;
  SendMessage(LoWord (Msg.lParam), tm_SetPrinterOffset, 0, Longint (@NewOffset[0]));
end;

{----------------------------------------------------------------------}
{----------------------------------------------------------------------}
{----------------------------------------------------------------------}

function THighEdit.GetBkColor: TColor;
begin
  result := FBkBrush.Color;
end;

{----------------------------------------------------------------------}

function THighEdit.GetFileName: TFileName;
var
  Name: array [0..255] of Char;
begin
  StrCopy(Name, FFileBoxParamPtr^.szPathName);
  StrCat(Name, FFileBoxParamPtr^.szFileName);
  result := StrPas(Name);
end;

{----------------------------------------------------------------------}

function THighEdit.GetGrayBk: Boolean;
begin
  Result := HEIsGrayBk;
end;

{----------------------------------------------------------------------}

function THighEdit.GetShowControls: TShowControls;
begin
  Result := [];
  if FUseIcon then Include(Result, scIconBar);
  if FUseRuler then Include(Result, scRuler);
  if FUseTab then Include(Result, scTabulator);
  if FUseStatus then Include(Result, scStatusLine);
end;

{----------------------------------------------------------------------}

function THighEdit.GetUnitType: TUnitType;
var
  UnitType: Longint;
begin
  UnitType := HEGetUnit;
  if UnitType = unit_CM
  then Result := utCentimeter
  else Result := utInch;
end;

{----------------------------------------------------------------------}

function THighEdit.GetVersion: LongInt;
begin
  Result := HighEditVersion;
end;

{----------------------------------------------------------------------}

procedure THighEdit.ChangeStatusFont(Sender: TObject);
begin
  if FUseStatus then begin
    CreateStatusLine;
    Resize;
    Refresh;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.SetBkColor(Value: TColor);
begin
  FBkBrush.Color := Value;
  HESetPFBkColor(Value);
  GrayBk := false;
end;

{----------------------------------------------------------------------}

procedure THighEdit.SetFileName(Value: TFileName);
var
  Ext: String[4];
begin
  if CanClose then begin
    with FFileBoxParamPtr^ do begin
      StrPCopy(szPathName, ExtractFilePath(Value));
      StrPCopy(szFileName, ExtractFileName(Value));
      Ext := LowerCase(ExtractFileExt(Value));
      StrLower(szPathName);
      StrLower(szFileName);
    end;
    if Value <> ''
    then begin
      if Ext = '.txt' then
         FFileBoxParamPtr^.wFileType := FileFormat_ANSI
      else
         if Ext = '.rtf' then
            FFileBoxParamPtr^.wFileType := FileFormat_RTF
         else
            FFileBoxParamPtr^.wFileType := FileFormat_HighEdit;
      if (HELoad) then
        FUntitled := FALSE;
        Modified := FALSE;
    end
    else HENew;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.SetGrayBk(Value: Boolean);
begin
  HESetGrayBk(Value);
end;

{----------------------------------------------------------------------}

procedure THighEdit.SetShowControls(Value: TShowControls);
begin
  if scIconBar in Value then begin
    if not FUseIcon then begin
      FUseIcon := True;
      if not (csLoading in ComponentState) then begin
        CreateIconBar;
        Resize;
        Refresh;
      end;
    end;
  end
  else begin
    if FUseIcon then begin
      FUseIcon := False;
      if not (csLoading in ComponentState) then begin
        DestroyWindow(FIconHandle);
        FIconHandle := 0;
        Resize;
        Refresh;
      end;
    end;
  end;

  if scRuler in Value then begin
    if not FUseRuler then begin
      FUseRuler := True;
      if not (csLoading in ComponentState) then begin
        CreateRuler;
        Resize;
        Refresh;
      end;
    end;
  end
  else begin
    if FUseRuler then begin
      FUseRuler := False;
      if not (csLoading in ComponentState) then begin
        DestroyWindow(FRulerHandle);
        FRulerHandle := 0;
        Resize;
        Refresh;
      end;
    end;
  end;

  if scTabulator in Value then begin
    if not FUseTab then begin
      FUseTab := True;
      if not (csLoading in ComponentState) then begin
        CreateTabulator;
        Resize;
        Refresh;
      end;
    end;
  end
  else begin
    if FUseTab then begin
      FUseTab := False;
      if not (csLoading in ComponentState) then begin
        DestroyWindow(FTabHandle);
        FTabHandle := 0;
        Resize;
        Refresh;
      end;
    end;
  end;

  if scStatusLine in Value then begin
    if not FUseStatus then begin
      FUseStatus := True;
      if not (csLoading in ComponentState) then begin
        CreateStatusLine;
        Resize;
        Refresh;
      end;
    end;
  end
  else begin
    if FUseStatus then begin
      FUseStatus := False;
      if not (csLoading in ComponentState) then begin
        DestroyWindow(FStatusHandle);
        FStatusHandle := 0;
        Resize;
        Refresh;
      end;
    end;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.SetStatusFont(Value: TFont);
begin
  Value.Height := Value.Size;
  FStatusFont.Assign(Value);
end;

{----------------------------------------------------------------------}

procedure THighEdit.SetTextBorderStyle(Value: TBorderStyle);
var
  HasGrayBk: Boolean;
begin
  if FTextBorderStyle <> Value
  then begin
    if FHighEditHandle <> 0
    then HasGrayBk := GrayBk
    else HasGrayBk := false;
    FTextBorderStyle := Value;
    DestroyWindow(FHighEditHandle);
    CreateHighEdit;
    if HasGrayBk then GrayBk := true;
    Resize;
  end;
end;

{----------------------------------------------------------------------}

procedure THighEdit.SetTextSelectable(Value: Boolean);
begin
  FTextSelectable := Value;
  HESetTextSelectable(Value);
end;

{----------------------------------------------------------------------}

procedure THighEdit.SetUnitType(Value: TUnitType);
begin
  if Value = utCentimeter
  then UnitCM
  else UnitIN;
end;

{======================================================================}

initialization
  if hprevInst=0 then begin
    HighEditInit(HInstance);
    HEDialogInit(HInstance);
    HEIconBarInit(HInstance);
    HEToolsInit(HInstance);
  end;
end.

