{*******************************************************************}
{								    }
{	HECompDB Unit. Enthlt die Komponente TDBHighEdit, die      }
{	ihre Daten aus einen Binren Blob erhlt, bzw in diesen     }
{       schreiben kann                                              }
{	Name:  HECompDB.pas 					    }
{	Autor: Jan Hamann / Martin Kammann		            }
{	Datum: 5.4.94						    }
{								    }
{*******************************************************************}

unit Hecompdb;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, HEComp, DB, DBTables;

type
  TDBHighEdit = class(THighEdit)
  private
    FAutoDisplay            : Boolean;
    FDataLink               : TFieldDataLink;
    FTextLoaded             : Boolean;
    procedure Change;
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetReadOnly: Boolean;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
    procedure SetAutoDisplay(Value: Boolean);
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetReadOnly(Value: Boolean);
    procedure UpdateData(Sender: TObject);
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure LoadText;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LoadFromBlob(ABlobField: TBlobField);
    procedure SaveToBlob(ABlobField: TBlobField);
    procedure StartEdit;
  published
    property AutoDisplay: Boolean read FAutoDisplay write SetAutoDisplay default True;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
  end;

implementation

constructor TDBHighEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FAutoDisplay := True;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
end;


destructor TDBHighEdit.Destroy;
begin
  FDataLink.OnDataChange := nil;
  FDataLink.OnEditingChange := nil;
  FDataLink.OnUpdateData := nil;
  FDataLink.Free;
  FDataLink := nil;
  Inherited Destroy;
end;


procedure TDBHighEdit.Change;
begin
  FDataLink.Modified;
  FTextLoaded := True;
end;


procedure TDBHighEdit.DataChange(Sender: TObject);
begin
  if FDataLink.Field <> nil then
    if FDataLink.Field is TBlobField then
      if FAutoDisplay or (FDataLink.Editing and FTextLoaded) then
      begin
        FTextLoaded := False;
        LoadText;
      end
      else FTextLoaded := False
    else FTextLoaded := False
  else FTextLoaded := False;
end;

procedure TDBHighEdit.EditingChange(Sender: TObject);
begin
  HESetReadOnly(not (FDataLink.Editing and FTextLoaded));
  HESetTextSelectable(True);
end;

procedure TDBHighEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if DataSource <> nil then begin
    if FTextLoaded then
    begin
      if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then
        FDataLink.Edit;
    end else
      Key := 0;
  end;
end;


procedure TDBHighEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if DataSource <> nil then begin
    if FTextLoaded then
    begin
      if (Key in [#32..#255]) and (FDataLink.Field <> nil) and
        not FDataLink.Field.IsValidChar(Key) then
      begin
        MessageBeep(0);
        Key := #0;
      end;
      case Key of
        ^H, ^I, ^J, ^M, ^V, ^X, #32..#255: begin
          FDataLink.Edit;
          FDataLink.Modified;
          end;
        #27:
          FDataLink.Reset;
      end;
    end else
    begin
      if Key = #13 then LoadText;
      Key := #0;
    end;
  end;
end;

procedure TDBHighEdit.LoadFromBlob(ABlobField: TBlobField);
var
  S: TBlobStream;
  hBuf: THandle;
  lpBuf: Pointer;
  Count: LongInt;
begin
  try
    Redraw := False;
    HENew;
    S := TBlobStream.Create(ABlobField, bmRead);
    if S.Size > 0 then begin
       hBuf := GlobalAlloc(GPTR, S.Size);
       lpBuf := GlobalLock(hBuf);
       try
         Count := S.Read(lpBuf^, S.Size);
         HEPasteRTF(hBuf);
       finally
         GlobalUnlock(hBuf);
         hBuf := GlobalFree(hBuf);
       end;
    end;
  finally
    Redraw := True;
    S.Free;
    FTextLoaded := True;
  end;
end;

procedure TDBHighEdit.LoadText;
begin
  if not FTextLoaded and (FDataLink.Field is TBlobField) then
  begin
    LoadFromBlob(FDataLink.Field as TBlobField);
    EditingChange(Self);
  end;
end;

procedure TDBHighEdit.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TDBHighEdit.SaveToBlob(ABlobField: TBlobField);
var
  S: TBlobStream;
  Size: LongInt;
  hBuf: THandle;
  lpBuf: Pointer;
  Count: LongInt;
begin
  try
    S := TBlobStream.Create(ABlobField, bmWrite);
    Redraw := False;
    EditSelect;
    hBuf := HECopyRTF;
    if hBuf <> 0 then begin
      Size := GlobalSize(hBuf);
      lpBuf := GlobalLock(hBuf);
      Count := S.Write(lpBuf^, Size);
      GlobalUnlock(hBuf);
      GlobalFree(hBuf);
    end;
  finally
    HESetSelection(0, 0);
    Redraw := True;
    S.Free;
  end;
end;

procedure TDBHighEdit.StartEdit;
begin
  if FTextLoaded and (FDataLink.Field <> nil) then begin
    FDataLink.Edit;
    FDataLink.Modified;
  end;
end;

procedure TDBHighEdit.UpdateData(Sender: TObject);
begin
  if FDataLink.Field is TBlobField then begin
    SaveToBlob(FDataLink.Field as TBlobField);
  end;
end;

function TDBHighEdit.GetDataField: String;
begin
  Result := FDataLink.FieldName;
end;

function TDBHighEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TDBHighEdit.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TDBHighEdit.SetAutoDisplay(Value: Boolean);
begin
  if FAutoDisplay <> Value then
  begin
    FAutoDisplay := Value;
    if Value then LoadText;
  end;
end;

procedure TDBHighEdit.SetDataField(const Value: String);
begin
  FDataLink.FieldName := Value;
end;

procedure TDBHighEdit.SetDataSource(Value: TDataSource);
begin
  FDataLInk.DataSource := Value;
end;

procedure TDBHighEdit.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;




end.
